
/*==============================================================================
 * (#)Help.java
 * created      2003/05/07
 * version      $Revision: 1.1.1.1 $Date: 2003/10/18 03:17:49 $
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *==============================================================================
 */


package com.miami.romeo.help;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import javax.swing.JFrame;
import javax.swing.JEditorPane;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.ImageIcon;
import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.tree.TreeSelectionModel;
import javax.swing.tree.TreePath;

import java.io.*;
import java.io.IOException;

import java.util.*;

import java.net.URL;

import com.miami.romeo.config.*;
import com.miami.romeo.internet.*;

public class Help extends JFrame {

  Utils utils = new Utils();

  JMenuBar menuBar;
  // Tool Bar
  private ToggleButtonToolBar toolbar = null;
  private ButtonGroup toolbarGroup = new ButtonGroup();
  final JTree tree;
  private JEditorPane htmlPane;
  private static boolean DEBUG = false;
  private URL helpURL;
  private boolean playWithLineStyle = false;
  private String lineStyle = "Angled";

  String[] helpUrls = {"docs.html",
                       "about.html",
		       "installation.html",
		       "settings.html",
		       "tabs.html"};

  Vector history;

  Mailer mailer = null;
  Downloader downloader = null;

  public Help() {
    JPanel top0 = new JPanel();
    top0.setLayout(new BorderLayout());
    this.getContentPane().add(top0, BorderLayout.NORTH);

    // Add the JMenuBar.
    menuBar = createMenus();
    top0.add(menuBar, BorderLayout.NORTH);

    // Add the ToolBar.
    JPanel toolbarPanel = new JPanel();
    toolbarPanel.setLayout(new BorderLayout());
    toolbar = new ToggleButtonToolBar();
    toolbarPanel.add(toolbar, BorderLayout.NORTH);
    createToggleButtons();
    top0.add(toolbarPanel, BorderLayout.SOUTH);
    getToolBar().getComponentAtIndex(0).setEnabled(false);

    // Create nodes
    DefaultMutableTreeNode top = new DefaultMutableTreeNode("Help");
    createNodes(top);

    // Create a jtree with single selection
    tree = new JTree(top);
    tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);

    // Set icons for the nodes
    DefaultTreeCellRenderer renderer = new DefaultTreeCellRenderer();
    renderer.setLeafIcon(utils.createImageIcon16x16("LeafHelp.png"));
    renderer.setOpenIcon(utils.createImageIcon16x16("Help.png"));
    renderer.setClosedIcon(utils.createImageIcon16x16("ClosedHelp.png"));
    tree.setCellRenderer(renderer);

    // Tree selection listener
    tree.addTreeSelectionListener(new TreeSelectionListener() {
      public void valueChanged(TreeSelectionEvent e) {
        DefaultMutableTreeNode node = (DefaultMutableTreeNode)
                                      tree.getLastSelectedPathComponent();
        if (node == null) return;

        Object nodeInfo = node.getUserObject();
        if (node.isLeaf()) {
          BookInfo book = (BookInfo)nodeInfo;
          displayURL(book.bookURL);
          if (DEBUG) {
            System.out.println(book.bookURL + ":  \n  ");
          }
        } else {
            displayURL(helpURL);
        }
        if (DEBUG) {
          System.out.println(nodeInfo.toString());
        }
      }
    });

    if (playWithLineStyle) {
      tree.putClientProperty("JTree.lineStyle", lineStyle);
    }

    // Create a scroll pane and add the tree
    JScrollPane treeView = new JScrollPane(tree);

    // Create the viewer
    htmlPane = new JEditorPane();
    htmlPane.setEditable(false);
    htmlPane.addHyperlinkListener(new HyperlinkListener() {
      public void hyperlinkUpdate(HyperlinkEvent r) {
        try {
          if (r.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
	    String urlToString = r.getURL().toString();

	    int idx_slash = urlToString.lastIndexOf('/');

            if (urlToString.endsWith(helpUrls[0]) ||
	        urlToString.endsWith(helpUrls[1]) ||
		urlToString.endsWith(helpUrls[2]) ||
		urlToString.substring(idx_slash + 1).startsWith(helpUrls[2]) ||
		urlToString.endsWith(helpUrls[3]) ||
		urlToString.endsWith(helpUrls[4]) ||
		urlToString.substring(idx_slash + 1).startsWith(helpUrls[4])) {
              htmlPane.setPage(r.getURL());

	      setChangePageButtonsEnabled();

	      int visibleUrl = getVisibleUrl();

	      Object[] path = new Object[2];
              path[0] = tree.getModel().getRoot();
              path[1] = tree.getModel().getChild(path[0], visibleUrl);
              tree.setSelectionPath(new TreePath(path));
	    }
	    else if (r.getURL().toString().endsWith(".rpm") ||
                     r.getURL().toString().endsWith(".tar.bz2") ||
		     r.getURL().toString().endsWith(".tar.gz") ||
		     r.getURL().toString().endsWith(".zip") ||
		     r.getURL().toString().endsWith(".dat") ||
		     r.getURL().toString().endsWith(".diff")) {
	      // Create a file chooser
              JFileChooser fc = new JFileChooser();
	      fc.setSelectedFile(new File(r.getURL().toString()));
              int returnVal = fc.showSaveDialog(Help.this);
              if (returnVal == JFileChooser.APPROVE_OPTION) {
                String path = fc.getSelectedFile().getPath();
	        int idx = path.lastIndexOf(File.separator);
	        String destination = path.substring(0, idx + 1);
	        downloader = new Downloader(r.getURL().toString().substring(
		                            0, r.getURL().toString().lastIndexOf('/') + 1),
		                            destination, r.getURL().toString().substring(
		                            r.getURL().toString().lastIndexOf('/') + 1));
              }
	    }
	    else if (r.getURL().toString().endsWith("stef_bol@libero.it")) {
	      if (mailer == null)
	        mailer = new Mailer();
	      else
	        mailer.setVisible(true);
	    }
	    else {
	      htmlPane.setPage(r.getURL());

	      getJMenuBar().getMenu(1).getItem(0).setEnabled(false);
	      getJMenuBar().getMenu(1).getItem(1).setEnabled(false);
	      getToolBar().getComponentAtIndex(0).setEnabled(false);
	      getToolBar().getComponentAtIndex(1).setEnabled(false);
	    }
	  }
        }
        catch (Exception e) {
	  System.err.println(e.getMessage());
        }
      }
    });
    initHelp();
    JScrollPane htmlView = new JScrollPane(htmlPane);

    // Add scroll panes to the split pane
    JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
    splitPane.setLeftComponent(treeView);
    splitPane.setRightComponent(htmlView);

    Dimension minimumSize = new Dimension(100, 300);
    htmlView.setMinimumSize(minimumSize);
    treeView.setMinimumSize(minimumSize);
    splitPane.setDividerLocation(150);

    splitPane.setPreferredSize(new Dimension(500, 300));

    // Add the split pane to the frame
    this.getContentPane().add(splitPane, BorderLayout.CENTER);

    JPanel statusBarPanel = new JPanel();
    statusBarPanel.setLayout(new BorderLayout());
    JTextField statusField = new JTextField();
    statusField.setEditable(false);
    statusBarPanel.add(statusField, BorderLayout.CENTER);

    this.getContentPane().add(statusBarPanel, BorderLayout.SOUTH);

    this.setTitle("Help");
    Image icnHelp = Toolkit.getDefaultToolkit().getImage(
                           getClass().getResource("/images/icons/16x16/Help.png"));
    this.setIconImage(icnHelp);
    this.setSize(800, 600);
    Dimension screenDim = Toolkit.getDefaultToolkit().getScreenSize();
    Rectangle frameDim = getBounds();
    this.setLocation((screenDim.width - frameDim.width) / 2,
                     (screenDim.height - frameDim.height) / 2);

        this.setVisible(true);
  }

  /**
   * Create menus
   */
  public JMenuBar createMenus() {
    JMenuItem mi;
    // ***** create the menubar ****
    Border borderMenuBar = BorderFactory.createBevelBorder(BevelBorder.RAISED,
                                                    new Color(248, 254, 255),
                                                    Color.white,
                                                    new Color(121, 124, 136),
                                                    new Color(84, 86, 95));
    JMenuBar menuBar = new JMenuBar();
    menuBar.setBorder(borderMenuBar);

    // ***** create File menu
    JMenu fileMenu = (JMenu) menuBar.add(new JMenu(utils.getString("FileMenu.file_label")));
    fileMenu.setMnemonic(utils.getMnemonic("FileMenu.file_mnemonic"));

    createMenuItem(fileMenu, "FileMenu.exit_label", "FileMenu.exit_mnemonic",
		   "FileMenu.exit_accessible_description", "Exit.png",
		   new ExitAction(this));

    // ***** create Go menu
    JMenu goMenu = (JMenu) menuBar.add(new JMenu(utils.getString("GoMenu.file_label")));
    fileMenu.setMnemonic(utils.getMnemonic("GoMenu.file_mnemonic"));

    createMenuItem(goMenu, "GoMenu.previous_label", "GoMenu.previous_mnemonic",
		   "GoMenu.previous_accessible_description", "Back.png",
		   new PreviousPageAction(this));

    createMenuItem(goMenu, "GoMenu.next_label", "GoMenu.next_mnemonic",
		   "GoMenu.next_accessible_description", "Forward.png",
		   new NextPageAction(this));

    return menuBar;
  }

  /**
   * Creates a generic JMenuItem
   */
  public JMenuItem createMenuItem(JMenu menu, String label, String mnemonic,
			          String accessibleDescription, String filename,
				  Action action) {
    JMenuItem mi = (JMenuItem) menu.add(new JMenuItem(utils.getString(label)));
    mi.setMnemonic(utils.getMnemonic(mnemonic));
    mi.getAccessibleContext().setAccessibleDescription(utils.getString(accessibleDescription));
    mi.setIcon(utils.createImageIcon16x16(filename));
    mi.addActionListener(action);
    if(action == null) {
      mi.setEnabled(false);
    }

    return mi;
  }

  /**
   * Creates JToggleButtons.
   */
  public void createToggleButtons() {
    String[] togglebuttons = {"Leftarrow",
                              "Rightarrow"};
    Action[] action = {new PreviousPageAction(this),
                       new NextPageAction(this)};
    for (int i = 0; i < togglebuttons.length; i++) {
      JToggleButton tb = this.getToolBar().addToggleButton(action[i],
                                           togglebuttons[i],
					   utils.createImageIcon24x24(togglebuttons[i] + ".png"),
					   utils.getString(togglebuttons[i] +
                                           ".togglebutton_accessible_description"));
      this.getToolBarGroup().add(tb);
      if(this.getToolBarGroup().getSelection() == null) {
        tb.setSelected(true);
      }
      tb.setText(null);
      tb.setToolTipText(utils.getString(togglebuttons[i] + ".togglebutton_accessible_description"));
    }
  }

  /**
   * Returns the menubar
   */
  public JMenuBar getJMenuBar() {
    return menuBar;
  }

  /**
   * Returns the toolbar
   */
  public ToggleButtonToolBar getToolBar() {
    return toolbar;
  }

  /**
   * Returns the toolbar button group
   */
  public ButtonGroup getToolBarGroup() {
    return toolbarGroup;
  }

  // *******************************************************
  // **************   ToggleButtonToolbar  *****************
  // *******************************************************

  static Insets zeroInsets = new Insets(1,1,1,1);
  protected class ToggleButtonToolBar extends JToolBar {
    public ToggleButtonToolBar() {
      super();
    }

    JToggleButton addToggleButton(Action a, String tbName,
                                  ImageIcon tbIcon, String tbToolTipText) {
      JToggleButton tb = new JToggleButton(tbName, tbIcon);
      tb.setMargin(zeroInsets);
      tb.setText(null);
      tb.setEnabled(a.isEnabled());
      tb.setToolTipText(utils.getString(tbToolTipText));
      tb.setAction(a);
      tb.setIcon(tbIcon);
      add(tb);
      return tb;
    }
  }

  private class BookInfo {

    public String bookName;
    public URL bookURL;
    public String prefix = "/html/Help/";

    public BookInfo(String book, String filename) {
      bookName = book;
      try {
        bookURL = getClass().getResource(prefix + filename);
      }
      catch (Exception exc) {
        System.out.println("\n");
        System.err.println("Error: creation failed");
        bookURL = null;
      }
    }

    public String toString() {
      return bookName;
    }
  }

  private void initHelp() {
    String s = null;
    try {
      s = "/html/Help/docs.html";

      if (DEBUG) {
        System.out.println("The Help URL is " + s);
      }
      helpURL = getClass().getResource(s);
      displayURL(helpURL);
    }
    catch (Exception e) {
      System.out.println("\n");
      System.err.println("Could not create the Help URL: " + s);
    }
  }

  private void displayURL(URL url) {
    try {
      htmlPane.setPage(url);

      setChangePageButtonsEnabled();
    }
    catch (IOException e) {
      System.out.println("\n");
      System.err.println("Could not read the URL: " + url);
    }
  }

  private void createNodes(DefaultMutableTreeNode top) {
    DefaultMutableTreeNode category = null;
    DefaultMutableTreeNode book = null;

    category = new DefaultMutableTreeNode(new BookInfo("Table of Contents",
                                          helpUrls[0]));
    top.add(category);

    category = new DefaultMutableTreeNode(new BookInfo("About Romeo",
                                          helpUrls[1]));
    top.add(category);

    category = new DefaultMutableTreeNode(new BookInfo("Installation",
                                          helpUrls[2]));

    top.add(category);

    category = new DefaultMutableTreeNode(new BookInfo("Configure Romeo",
                                          helpUrls[3]));
    top.add(category);

    category = new DefaultMutableTreeNode(new BookInfo("Getting Started",
                                          helpUrls[4]));
    top.add(category);
  }

  /**
   * This method locks/unlocks tool bar buttons
   */
  private void setChangePageButtonsEnabled() {
    if (htmlPane.getPage().toString().endsWith(helpUrls[0])) {
      if (getToolBar().getComponentAtIndex(0).isEnabled()) {
        getToolBar().getComponentAtIndex(0).setEnabled(false);
        getJMenuBar().getMenu(1).getItem(0).setEnabled(false);
      }

      if (!getToolBar().getComponentAtIndex(1).isEnabled()) {
        getToolBar().getComponentAtIndex(1).setEnabled(true);
	getJMenuBar().getMenu(1).getItem(1).setEnabled(true);
      }
    }
    else if (htmlPane.getPage().toString().endsWith(helpUrls[4])) {
      if (!getToolBar().getComponentAtIndex(0).isEnabled()) {
        getToolBar().getComponentAtIndex(0).setEnabled(true);
	getJMenuBar().getMenu(1).getItem(0).setEnabled(true);
      }

      if (getToolBar().getComponentAtIndex(1).isEnabled()) {
        getToolBar().getComponentAtIndex(1).setEnabled(false);
	getJMenuBar().getMenu(1).getItem(1).setEnabled(false);
      }
    }
    else {
      if (!getToolBar().getComponentAtIndex(0).isEnabled()) {
        getToolBar().getComponentAtIndex(0).setEnabled(true);
	getJMenuBar().getMenu(1).getItem(0).setEnabled(true);
      }

      if (!getToolBar().getComponentAtIndex(1).isEnabled()) {
        getToolBar().getComponentAtIndex(1).setEnabled(true);
	getJMenuBar().getMenu(1).getItem(1).setEnabled(true);
      }
    }
  }

  // *******************************************************
  // ********************   Actions  ***********************
  // *******************************************************



  /**
   * Previous Page Action
   */
  class PreviousPageAction extends AbstractAction {
    Help help;

    protected PreviousPageAction(Help help) {
      super("PreviousPageAction");
      this.help = help;
    }

    public void actionPerformed(ActionEvent e) {
      int visibleUrl = getVisibleUrl();

      String s = null;

      try {
        if (visibleUrl != -1) {
          s = "/html/Help/" + helpUrls[visibleUrl - 1];

          if (DEBUG) {
           System.out.println("The Help URL is " + s);
          }

          helpURL = getClass().getResource(s);
          displayURL(helpURL);

	  //select a path
          Object[] path = new Object[2];
          path[0] = tree.getModel().getRoot();
          path[1] = tree.getModel().getChild(path[0], visibleUrl - 1);
          tree.setSelectionPath(new TreePath(path));

	}
      }
      catch (Exception e1) {
        System.out.println("\n");
        System.err.println("Could not create the Help URL: " + s);
      }
    }
  }

  /**
   * Next Page Action
   */
  class NextPageAction extends AbstractAction {
    Help help;

    protected NextPageAction(Help help) {
      super("NextPageAction");
      this.help = help;
    }

    public void actionPerformed(ActionEvent e) {
      int visibleUrl = getVisibleUrl();

      String s = null;

      try {
        if (visibleUrl != -1) {
          s = "/html/Help/" + helpUrls[visibleUrl + 1];

          if (DEBUG) {
           System.out.println("The Help URL is " + s);
          }

          helpURL = getClass().getResource(s);
          displayURL(helpURL);

	  //select a path
          Object[] path = new Object[2];
          path[0] = tree.getModel().getRoot();
          path[1] = tree.getModel().getChild(path[0], visibleUrl + 1);
          tree.setSelectionPath(new TreePath(path));
	}
      }
      catch (Exception e1) {
        System.out.println("\n");
        System.err.println("Could not create the Help URL: " + s);
      }
    }
  }

  private void highlightNode() {
    int visibleUrl = getVisibleUrl();

    //select a path
    Object[] path = new Object[2];
    path[0] = tree.getModel().getRoot();
    path[1] = tree.getModel().getChild(path[0], visibleUrl + 1);
    tree.setSelectionPath(new TreePath(path));
  }

  private int getVisibleUrl() {
    int i = -1;

    URL url = htmlPane.getPage();

    String urlToString = url.toString();

    int idx_slash = urlToString.lastIndexOf('/');

    if (urlToString.endsWith(helpUrls[0])) // docs.html
      i = 0;
    else if (urlToString.endsWith(helpUrls[1])) // about.html
      i = 1;
    else if (urlToString.substring(idx_slash + 1).startsWith(helpUrls[2]) ||
             urlToString.endsWith(helpUrls[2])) // installation.html
      i = 2;
    else if (urlToString.endsWith(helpUrls[3])) // settings.html
      i = 3;
    else if (urlToString.substring(idx_slash + 1).startsWith(helpUrls[4]) ||
             urlToString.endsWith(helpUrls[4])) // tabs.html
      i = 4;
    else
      i = -1;

    return i;
  }

  /**
   * Exit Action
   */
  class ExitAction extends AbstractAction {
    Help help;
    protected ExitAction(Help help) {
      super("ExitAction");
      this.help = help;
    }

    public void actionPerformed(ActionEvent e) {
      dispose();
    }
  }
} // That's all, folks :)




