
/*==============================================================================
 *
 * (#)Romeo.java
 * Created 2002/11/24
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 *  Copyright 1999-2002 Matthew Robinson and Pavel Vorobiev.
 *  All Rights Reserved.
 *
 *  ===================================================
 *  This program contains code from the book "Swing"
 *  2nd Edition by Matthew Robinson and Pavel Vorobiev
 *  http://www.spindoczine.com/sbe
 *  =================================================== 
 *
 *  The above paragraph must be included in full, unmodified
 *  and completely intact in the beginning of any source code
 *  file that references, copies or uses (in any way, shape
 *  or form) code contained in this file.
 *
 *==============================================================================
 */

package com.miami.romeo.main;

//****** imports
// Library used to skin Romeo.
import com.l2fprod.gui.*;
import com.l2fprod.gui.plaf.skin.*;
import com.l2fprod.util.*;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.util.*;
import java.io.*;
import java.beans.XMLEncoder;
import java.beans.EventHandler;
import java.net.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.text.*;

import javax.swing.plaf.metal.DefaultMetalTheme;
import javax.swing.plaf.metal.MetalLookAndFeel;

import javax.accessibility.*;
import javax.help.*; // New

import com.miami.romeo.config.*;
import com.miami.romeo.themes.*;
import com.miami.romeo.xmame.*;
import com.miami.romeo.internet.*;
import com.miami.romeo.settings.*;
import com.miami.romeo.help.*;
import com.miami.romeo.main.about.*;

/**
 * <p>Title:       Romeo</p>
 * <p>Description: This is a Java frontend for M.A.M.E.. The
 *                 mame ports supported by this application
 *                 are XMame and the Windows command line.
 *                 XMame is the Unix/X11 port of the mame 
 *                 project. This port makes mame available on 
 *                 *ix machines using the X11R6 X-Window System
 *		   (and SVGAlib/ggi/XF86-DGA/OpenGL/SDL too).</p>
 * <p>Copyright:   Copyright (c) 2002-2003 by Stefano Bolli.
 *                 All Rights Reserved.</p>
 * <p>Company:     </p>
 * <p>Created      2002/11/24</p>
 *
 * @author         $Author: stefanobolli $
 * @version        $Revision: 1.13 $Date: 2004/01/04 00:41:00 $
 */
public class Romeo extends JPanel implements KeyEventDispatcher
                                             /*, MouseListener*/ {
  //******** Classes needed by Romeo.java
  // This class contains some utilities
  Utils utils = new Utils();
  // Path to the configuration file
  private String config_name = System.getProperty("user.home") + 
                               File.separator + ".romeo4xmame" + 
			       File.separator + "Romeo.ini";
  // Configuration file
  private File config_file = new File(config_name);
  // Class to read and write configurations
  Config config = new Config(config_file);
  // This class dialogs with xmame
  Xmame xmame = null;
  // This class saves roms data
  DataIO dataio = new DataIO(this);

  // Possible Look & Feels
  private String mac     = "com.sun.java.swing.plaf.mac.MacLookAndFeel";
  private String metal   = "javax.swing.plaf.metal.MetalLookAndFeel";
  private String motif   = "com.sun.java.swing.plaf.motif.MotifLookAndFeel";
  private String windows = "com.sun.java.swing.plaf.windows.WindowsLookAndFeel";

  // The current Look & Feel
  private String currentLookAndFeel = metal;

  // Main frame size
  private int SIZE_X;
  private int SIZE_Y;

  // Startup location
  private int LOCATION_X;
  private int LOCATION_Y;

  // Status Bar
  public JTextField statusField   = null;
  public JTextField statusField2  = null;
  public JTextField statusField3  = null;
  public JTextField statusField4  = null;
  public JProgressBar progressBar = null;
  public JPanel statusPanel       = null;
  public JPanel statusbarPanel    = null;

  // Cursors
  private Cursor waitCursor    = new Cursor(Cursor.WAIT_CURSOR);
  private Cursor defaultCursor = new Cursor(Cursor.DEFAULT_CURSOR);

  // Components for the tabbed pane.
  public Games games     = null;
  public Info info       = null;
  public Options options = null;
  // Tabbed pane
  private JTabbedPane tabbedPane = null;

  //******** Other frames
  // Source code viewer
  Src src               = null;
  // A simple browser
  Hyperlink hyperlink   = null;
  // Mailer
  Mailer mailer = null;
  // Downloader window
  Downloader downloader = null;
  // Settings window
  Settings settings     = null;
  //****** About Box
  // Credits
  ScrollText scrollText      = null;
  // Memory monitor by SwingSet2 Team: I need this class
  // to know Romeo' s performances...
  MemoryMonitor mm = null;

  UpdateDialog updateDialog = null;

  private JDialog aboutBox   = null;

  // Tool Bars
  private ToggleButtonToolBar toolbar  = null;
  private ButtonGroup toolbarGroup     = new ButtonGroup();
  private JComboBox searchOptions      = null;
  private MemComboBox searchField      = null;
  private ToggleButtonToolBar toolbar2 = null;
  private ButtonGroup toolbarGroup2    = new ButtonGroup();
  private JRadioButton ignoreCaseBox   = null;
  private JRadioButton startsWithBox   = null;
  private JRadioButton endsWithBox     = null;
  private Vector searchResults         = null;
  int value                              = 0;
  private ButtonGroup searchGroup      = new ButtonGroup();
  // Focus manager
  DefaultFocusManager dfocus           = null;

  // Menus
  private JMenuBar menuBar            = null;
  private JMenu themesMenu            = null;
  private ButtonGroup lafMenuGroup    = new ButtonGroup();
  private ButtonGroup themesMenuGroup = new ButtonGroup();

  JPanel toolbarPanel = null;
  JPanel top          = null;

  // contentPane cache, saved from the application frame
  Container contentPane = null;

  // Used cause Romeo is an application.
  private static JFrame frame = null;

  // Timer
  int message = 0;
  int loop    = 0;
  private javax.swing.Timer timer;
  
  // The help : new!
  protected HelpSet helpSet;
  protected HelpBroker helpBroker;
  static final String HELPSETNAME = "jar:file:docs" + File.separator +
                                    "Help.jar!/Romeo.hs";

  /**
   * Romeo main. Called cause we' re an application.
   * This should never be invoked directly.
   *
   * @param args The command line arguments
   */
  public static void main(String[] args) {
    String version = "\nThis is Romeo-0.1.3pre4, running on " +
	             System.getProperty("os.name") + "/" +
		     System.getProperty("os.arch") + " (" +
		     System.getProperty("os.version") + ")";

    if (args.length > 0) {
      for (int i = 0; i < args.length; i++) {
        if (args[i].equals("-?") || args[i].equalsIgnoreCase("-H")) {
	  System.out.println(version);
          System.out.println("\nUsage: java -jar Romeo.jar [ options ]");
	  System.out.println("\nwhere options include:");
	  System.out.println("     -? -H     print this help message");
	  System.out.println("     -V        print product version");
          System.out.println("\n--\nStefano Bolli <stef_bol@libero.it>");
	}
        else if (args[i].equalsIgnoreCase("-V")) {
          System.out.println(version);
	}
	else {
	  System.out.println("Option \"" + args[i] + "\" not found");
	}
      }
    }
    else {
      // Run Romeo
      Romeo romeo = new Romeo();
    }
  }

  /**
   * The constructor initializes all components and shows the proggy.
   *
   * @see #printInfo()
   * @see #createFrame()
   * @see #initializeRomeo()
   * @see #showSplash()
   * @see #showRomeo()
   * @see com.miami.romeo.main.Games#initGames()
   * @see #checkNewRelease()
   */
  public Romeo() {
    // Print a few informations about Romeo
    printInfo();

    // Create the frame
    frame = createFrame();

    // setLayout(new BorderLayout());
    setLayout(new BorderLayout());

    // Initialize all components
    initializeRomeo();

    // Does the user want the splash screen?
    if (config.getBoolean("Show_splashscreen"))
      showSplash();

    // Show Romeo
    showRomeo();

    games.initGames();

    // Check if a new release is out!
    if (config.getBoolean("Romeo.New_Release"))
      checkNewRelease();
      
    //addMouseListener(this);
  }

  /**
   * This method prints a few human readable informations about Romeo.
   */
  public void printInfo() {
    System.out.println("\n");
    System.out.println("****************************************************************");
    System.out.println("*                  Romeo - M.A.M.E. frontend                   *");
    System.out.println("*           Copyright (C) 2002-2003 by Stefano Bolli           *");
    System.out.println("*                                                              *");
    System.out.println("*          M.A.M.E. - Multiple Arcade Machine Emulator         *");
    System.out.println("* Copyright (C) 1997-2003 by Nicola Salmoria and the MAME Team *");
    System.out.println("*            xmame port maintained by Lawrence Gold            *");
    System.out.println("****************************************************************");
    System.out.println("\n");
  }

  /**
   * This method creates and shows the splash screen.
   *
   * @since 0.1a
   */
  public void showSplash() {
    // Splash Screen
    SplashWindow sw;

    // Image to show
    Image splashIm;

    MediaTracker mt = new MediaTracker(this);
    splashIm = Toolkit.getDefaultToolkit().getImage(getClass().getResource(
                                             "/images/splashscreen/Romeo.png"));

    mt.addImage(splashIm, 0);
    try {
      mt.waitForID(0);
    }
    catch(InterruptedException ie){
    }

    sw = new SplashWindow(getFrame(), splashIm);

    try {
      // Show the splash screen for 5 seconds
      Thread.sleep(5000);
    }
    catch(InterruptedException ie){
      /* to handle */
    }

    sw.dispose();
  }

  /**
   * This method initializes all components.
   *
   * @see #createHelp()
   * @see #createMenus()
   * @see #createToggleButtons()
   * @see #createToggleButtons2()
   * @see #createStatusBar()
   * @see #addContextHelp()
   */
  public void initializeRomeo() {
    // Create help
    createHelp();

    top = new JPanel();
    top.setLayout(new BorderLayout());
    add(top, BorderLayout.NORTH);

    // Add the JMenuBar
    menuBar = createMenus();
    top.add(menuBar, BorderLayout.NORTH);
    getJMenuBar().getMenu(0).getItem(0).setEnabled(false);
    helpBroker.enableHelpOnButton(getJMenuBar().getMenu(6).getItem(0),
                                  "top", null);
    getJMenuBar().getMenu(6).getItem(1).addActionListener(new
                                      CSH.DisplayHelpAfterTracking(helpBroker));

    // Add the tool bars
    toolbarPanel = new JPanel();
    toolbarPanel.setLayout(new BorderLayout());
    toolbar = new ToggleButtonToolBar();
    createToggleButtons();
    if (config.getBoolean("View.toolbar1"))
      toolbarPanel.add(toolbar, BorderLayout.NORTH);
    getToolBar().getComponentAtIndex(0).setEnabled(false);
    toolbar2 = new ToggleButtonToolBar();
    createToggleButtons2();
    if (config.getBoolean("View.toolbar2"))
      toolbarPanel.add(toolbar2, BorderLayout.SOUTH);
    top.add(toolbarPanel, BorderLayout.SOUTH);

    // Add the tabbed pane
    tabbedPane = new JTabbedPane();
    add(tabbedPane, BorderLayout.CENTER);

    //******* Tabs
    // Single selection tree, list of games, combo box to select 
    // xmame executable to use
    games = new Games(this);
    tabbedPane.addTab(utils.getString("TabbedPane.games_label"),
                      utils.createImageIcon16x16("Rom.png"), games);
    // Game informations
    info = new Info(this);
    tabbedPane.addTab(utils.getString("TabbedPane.info_label"),
                      utils.createImageIcon16x16("Info.png"), info);
    // Game options
    options = new Options(this);
    tabbedPane.addTab(utils.getString("TabbedPane.options_label"),
                      utils.createImageIcon16x16("Settings.png"), options);
    // Lock this tab on startup (no game has been selected)
    setLocked(true, options);

    // Create and add the status bar
    statusbarPanel = createStatusBar();
    add(statusbarPanel, BorderLayout.SOUTH);
    if (!config.getBoolean("View.statusbar"))
      statusbarPanel.removeAll();

    addContextHelp();
  }
  
  /**
   * This method creates the help system. Since version 0.1.3pre1
   * the documentation is JavaHelp based.
   *
   * @since 0.1.3pre1
   */
  protected void createHelp() {
    URL url;
    try {
      url = new URL(HELPSETNAME);
      helpSet = new HelpSet(null, url);
      helpBroker = helpSet.createHelpBroker();
      helpBroker.enableHelpKey(getFrame().getRootPane(), "top", helpSet);
      
      //HistoryView hv = new HistoryView(helpSet, "HistoryView",
                                       //"HistoryView", new Hashtable(64));
    }
    catch (Exception ex) { ex.printStackTrace(); }
  }

  /**
   * This method creates and returns all menus.
   *
   * @see #createMenuItem(JMenu, String, String, String, String, Action)
   * @see #createCheckBoxMenuItem(JMenu, String, String, String, boolean, Action)
   * @see #createLafMenuItem(JMenu, String, String, String, String)
   * @see #createThemesMenuItem(JMenu, String, String, String, DefaultMetalTheme)
   * @see #createLinkMenuItem(JMenu, String, String)
   *
   * @return Menus
   */
  public JMenuBar createMenus() {
    JMenuItem mi;
    // ***** create the menubar ****
    JMenuBar menuBar = new JMenuBar();

    Border borderMenuBar = BorderFactory.createBevelBorder(BevelBorder.RAISED,
                                                    new Color(248, 254, 255),
                                                    Color.white,
                                                    new Color(121, 124, 136),
                                                    new Color(84, 86, 95));

    menuBar.setBorder(borderMenuBar);

    // ***** create File menu
    JMenu fileMenu = (JMenu) menuBar.add(new JMenu(utils.getString(
                                                   "FileMenu.file_label")));
    fileMenu.setMnemonic(utils.getMnemonic("FileMenu.file_mnemonic"));

      mi = createMenuItem(fileMenu, "FileMenu.play_label",
                          "FileMenu.play_mnemonic",
	  	          "FileMenu.play_accessible_description",
                          "Exec.png",
		          new PlayAction(this));

      fileMenu.addSeparator();

      mi = createMenuItem(fileMenu, "FileMenu.settings_label",
                          "FileMenu.settings_mnemonic",
	    	          "FileMenu.settings_accessible_description",
                          "Settings.png",
		          new SettingsAction(this));

      mi = createMenuItem(fileMenu, "FileMenu.reload_label",
                          "FileMenu.reload_mnemonic",
	     	          "FileMenu.reload_accessible_description",
                          "Reload.png",
		          new UpdateListOfGamesAction(this));

      fileMenu.addSeparator();

      mi = createMenuItem(fileMenu, "FileMenu.exit_label",
                          "FileMenu.exit_mnemonic",
	   	          "FileMenu.exit_accessible_description",
                          "Exit.png",
		          new ExitAction(this));

    // ***** create view menu
    JMenu viewMenu = (JMenu) menuBar.add(new JMenu(utils.getString(
                                                   "ViewMenu.view_label")));
    viewMenu.setMnemonic(utils.getMnemonic("ViewMenu.view_mnemonic"));

    JMenu viewToolBarsMenu = (JMenu) viewMenu.add(new JMenu(utils.getString(
                                        "ViewMenu.viewtoolbarsmenu_label")));
    viewToolBarsMenu.setMnemonic(utils.getMnemonic(
                                      "ViewMenu.viewtoolbarsmenu_mnemonic"));

      mi = createCheckBoxMenuItem(viewToolBarsMenu, "viewMenu.toolbar1_label",
                                  "viewMenu.toolbar1_mnemonic",
			          "viewMenu.toolbar1_accessible_description",
                                  config.getBoolean("View.toolbar1"),
				  new ViewToolBarAction(this, "first"));

      mi = createCheckBoxMenuItem(viewToolBarsMenu, "viewMenu.toolbar2_label",
                                  "viewMenu.toolbar2_mnemonic",
			          "viewMenu.toolbar2_accessible_description",
                                  config.getBoolean("View.toolbar2"),
				  new ViewToolBarAction(this, "second"));
				  
      mi = createCheckBoxMenuItem(viewMenu, "viewMenu.messageArea_label",
                                  "viewMenu.messageArea_mnemonic",
			          "viewMenu.messageArea_accessible_description",
                                  config.getBoolean("View.messageArea"),
				  new ViewMessageAreaAction(this));

      mi = createCheckBoxMenuItem(viewMenu, "viewMenu.statusbar_label",
                                  "viewMenu.statusbar_mnemonic",
			          "viewMenu.statusbar_accessible_description",
                                  config.getBoolean("View.statusbar"),
				  new ViewStatusBarAction(this));

    // ***** create laf switcher menu
    JMenu lafMenu = (JMenu) menuBar.add(new JMenu(utils.getString(
                                                  "LafMenu.laf_label")));
    lafMenu.setMnemonic(utils.getMnemonic("LafMenu.laf_mnemonic"));
    lafMenu.getAccessibleContext().setAccessibleDescription(
	    utils.getString("LafMenu.laf_accessible_description"));

      mi = createLafMenuItem(lafMenu, "LafMenu.java_label",
                             "LafMenu.java_mnemonic",
	  	             "LafMenu.java_accessible_description", metal);
      mi.setSelected(true); // this is the default l&f

      mi = createLafMenuItem(lafMenu, "LafMenu.mac_label",
                             "LafMenu.mac_mnemonic",
	  	             "LafMenu.mac_accessible_description", mac);

      mi = createLafMenuItem(lafMenu, "LafMenu.motif_label",
                             "LafMenu.motif_mnemonic",
		             "LafMenu.motif_accessible_description", motif);

      mi = createLafMenuItem(lafMenu, "LafMenu.windows_label",
                             "LafMenu.windows_mnemonic",
	                     "LafMenu.windows_accessible_description", windows);

    // ***** create themes menu
    themesMenu = (JMenu) menuBar.add(new JMenu(utils.getString(
                                                   "ThemesMenu.themes_label")));
    themesMenu.setMnemonic(utils.getMnemonic("ThemesMenu.themes_mnemonic"));
    themesMenu.getAccessibleContext().setAccessibleDescription(
               utils.getString("ThemesMenu.themes_accessible_description"));

      mi = createThemesMenuItem(themesMenu, "ThemesMenu.default_label",
                                "ThemesMenu.default_mnemonic",
		                "ThemesMenu.default_accessible_description",
                                new DefaultMetalTheme());
      mi.setSelected(true); // This is the default theme

      mi = createThemesMenuItem(themesMenu, "ThemesMenu.aqua_label",
                                "ThemesMenu.aqua_mnemonic",
	                        "ThemesMenu.aqua_accessible_description",
                                new AquaTheme());

      mi = createThemesMenuItem(themesMenu, "ThemesMenu.charcoal_label",
                                "ThemesMenu.charcoal_mnemonic",
		                "ThemesMenu.charcoal_accessible_description",
                                new CharcoalTheme());

      mi = createThemesMenuItem(themesMenu, "ThemesMenu.contrast_label",
                                "ThemesMenu.contrast_mnemonic",
	                        "ThemesMenu.contrast_accessible_description",
                                new ContrastTheme());

      mi = createThemesMenuItem(themesMenu, "ThemesMenu.emerald_label",
                                "ThemesMenu.emerald_mnemonic",
	    	                "ThemesMenu.emerald_accessible_description",
                                new EmeraldTheme());

      mi = createThemesMenuItem(themesMenu, "ThemesMenu.ruby_label",
                                "ThemesMenu.ruby_mnemonic",
		                "ThemesMenu.ruby_accessible_description",
                                new RubyTheme());

    // ***** create Internet menu
    JMenu internetMenu = (JMenu) menuBar.add(new JMenu(utils.getString(
                                               "InternetMenu.internet_label")));
    internetMenu.setMnemonic(utils.getMnemonic(
                                             "InternetMenu.internet_mnemonic"));

      mi = createMenuItem(internetMenu, "InternetMenu.rom_label",
                          "InternetMenu.rom_mnemonic",
		          "InternetMenu.rom_accessible_description",
                          "Rom.png", null);

      mi = createMenuItem(internetMenu, "InternetMenu.snap_label",
                          "InternetMenu.snap_mnemonic",
		          "InternetMenu.snap_accessible_description",
                          "Image.png", null);

      mi = createMenuItem(internetMenu, "InternetMenu.flyer_label",
                          "InternetMenu.flyer_mnemonic",
		          "InternetMenu.flyer_accessible_description",
                          "Image.png", null);

      mi = createMenuItem(internetMenu, "InternetMenu.marquee_label",
                          "InternetMenu.marquee_mnemonic",
		          "InternetMenu.marquee_accessible_description",
                          "Image.png", null);

      mi = createMenuItem(internetMenu, "InternetMenu.cabinet_label",
                          "InternetMenu.cabinet_mnemonic",
	  	          "InternetMenu.cabinet_accessible_description",
                          "Image.png", null);
			  
      mi = createMenuItem(internetMenu, "InternetMenu.sample_label",
                          "InternetMenu.sample_mnemonic",
	  	          "InternetMenu.sample_accessible_description",
                          "Tgz.png", null);
			  
      mi = createMenuItem(internetMenu, "InternetMenu.artwork_label",
                          "InternetMenu.artwork_mnemonic",
	  	          "InternetMenu.artwork_accessible_description",
                          "Tgz.png", null);
			  
    // ***** create Links menu
    JMenu linksMenu = (JMenu) menuBar.add(new JMenu(
                                        utils.getString("LinksMenu.links_label")));
    //linkMenu.setMnemonic();
    
      String[] linkLabel = {"Xmame/xmess Homepage",
                            "MAME - The official Multiple Arcade Machine Emulator site",
                            /*"[mame.dk] - arcade rom and information repository",*/
			    "CATVER - Category and Version Information for Mame",
			    "history.dat - Tips / tricks and game history information",
			    "cheat.dat -  File to enable cheating in the games"};

      String[] address = {"http://x.mame.net/",
                          "http://www.mame.net/",
                          /*"http://www.mame.dk/",*/
			  "http://www.catver.com/",
			  "http://fandemame.emu-france.com/",
			  "http://cheat.retrogames.com/"};

      for (int i = 0; i < linkLabel.length; i++) {
        mi = createLinkMenuItem(linksMenu, linkLabel[i], address[i]);
	
	if (i == 2)
          linksMenu.addSeparator();
      }

    // ***** create Help menu
    JMenu helpMenu = (JMenu) menuBar.add(new JMenu(utils.getString(
                                                       "HelpMenu.help_label")));
    helpMenu.setMnemonic(utils.getMnemonic("HelpMenu.help_mnemonic"));

      mi = createMenuItem(helpMenu, "HelpMenu.content_label",
                          "HelpMenu.content_mnemonic",
		          "HelpMenu.content_accessible_description", "Help.png",
                          null);
      mi.setEnabled(true);

      mi = createMenuItem(helpMenu, "HelpMenu.context_label",
                          "HelpMenu.context_mnemonic",
		          "HelpMenu.context_accessible_description", "Contexthelp.png",
                          null);
      mi.setEnabled(true);

      helpMenu.addSeparator();

      mi = createMenuItem(helpMenu, "HelpMenu.src_label", "HelpMenu.src_mnemonic",
		          "HelpMenu.src_accessible_description", "Java_src.png",
                          new SrcAction(this));

      helpMenu.addSeparator();

      mi = createMenuItem(helpMenu, "HelpMenu.web_label", "HelpMenu.web_mnemonic",
		          "HelpMenu.web_accessible_description", "Network.png",
                          new HomepageAction(this));

      mi = createMenuItem(helpMenu, "HelpMenu.release_label",
                          "HelpMenu.release_mnemonic",
		          "HelpMenu.release_accessible_description", "Romeo.png",
                          new ReleaseAction(this));

      mi = createMenuItem(helpMenu, "HelpMenu.mail_label",
                          "HelpMenu.mail_mnemonic",
	  	          "HelpMenu.mail_accessible_description",
                          "Mail_generic.png", new MailAction(this));

      helpMenu.addSeparator();

      mi = createMenuItem(helpMenu, "HelpMenu.about_label",
                          "HelpMenu.about_mnemonic",
	   	          "HelpMenu.about_accessible_description", "Info.png",
                          new AboutAction(this));

    return menuBar;
  }

  /**
   * This method creates a generic JMenuItem.
   *
   * @param menu                  The menu to add the menu item to
   * @param label                 The menu item' s text
   * @param mnemonic              The keyboard mnemonic on the current component
   * @param accessibleDescription The accessible description of this object
   * @param filename              The icon this component will display
   * @param action                The Action for the ActionEvent source
   *
   * @return                      A <code>JMenuItem</code> for the menu
   */
  public JMenuItem createMenuItem(JMenu menu, String label, String mnemonic,
			          String accessibleDescription, String filename,
				  Action action) {
    JMenuItem mi = (JMenuItem) menu.add(new JMenuItem(utils.getString(label)));
    mi.setMnemonic(utils.getMnemonic(mnemonic));
    mi.getAccessibleContext().setAccessibleDescription(utils.getString(
                                                        accessibleDescription));
    mi.setIcon(utils.createImageIcon16x16(filename));
    mi.addActionListener(action);
    if(action == null) {
      mi.setEnabled(false);
    }

    return mi;
  }

  /**
   * This method creates a generic JCheckBoxMenuItem.
   *
   * @param menu                  The menu to add the JCheckBoxMenuItem to
   * @param label                 The components' s text
   * @param mnemonic              The keyboard mnemonic on the current component
   * @param accessibleDescription The accessible description of this object
   * @param selected              True if the component is selected, false otherwise
   * @param action                The Action for the ActionEvent source
   *
   * @return                      A <code>JCheckBoxMenuItem</code> for the menu
   */
  public JMenuItem createCheckBoxMenuItem(JMenu menu, String label,
                                          String mnemonic,
			                  String accessibleDescription,
                                          boolean selected,
					  Action action) {
    JMenuItem mi = (JCheckBoxMenuItem) menu.add(new JCheckBoxMenuItem(
                                                       utils.getString(label)));
    mi.setMnemonic(utils.getMnemonic(mnemonic));
    mi.getAccessibleContext().setAccessibleDescription(utils.getString(
                                                        accessibleDescription));
    mi.addActionListener(action);
    if(action == null) {
      mi.setEnabled(false);
    }

    try {
      mi.setSelected(selected);
    }
    catch (Exception e) {
      System.err.println(e.toString());
    }

    return mi;
  }

  /**
   * This method creates a JRadioButtonMenuItem for the Look and Feel menu.
   *
   * @param menu                  The menu to add the JRadioButtonMenuItem to
   * @param label                 The components' s text
   * @param mnemonic              The keyboard mnemonic on the current component
   * @param accessibleDescription The accessible description of this object
   * @param laf                   The look and feel
   *
   * @return                      A <code>JRadioButtonMenuItem</code> for the LnF menu
   */
  public JMenuItem createLafMenuItem(JMenu menu, String label, String mnemonic,
                                     String accessibleDescription, String laf) {
    JMenuItem mi = (JRadioButtonMenuItem) menu.add(new JRadioButtonMenuItem(
                                                       utils.getString(label)));
    lafMenuGroup.add(mi);
    mi.setMnemonic(utils.getMnemonic(mnemonic));
    mi.getAccessibleContext().setAccessibleDescription(utils.getString(
                                                        accessibleDescription));
    mi.addActionListener(new ChangeLookAndFeelAction(this, laf));

    mi.setEnabled(isAvailableLookAndFeel(laf));

    return mi;
  }

  /**
   * This method creates a JRadioButtonMenuItem for the Themes menu.
   *
   * @param menu                  The menu to add the JRadioButtonMenuItem to
   * @param label                 The components' s text
   * @param mnemonic              The keyboard mnemonic on the current component
   * @param accessibleDescription The accessible description of this object
   * @param theme                 The theme to load
   *
   * @return                      A <code>JRadioButtonMenuItem</code> for the Themes menu
   */
  public JMenuItem createThemesMenuItem(JMenu menu, String label,
                                        String mnemonic,
			                String accessibleDescription,
                                        DefaultMetalTheme theme) {
   JRadioButtonMenuItem mi = (JRadioButtonMenuItem) menu.add(
                              new JRadioButtonMenuItem(utils.getString(label)));
   themesMenuGroup.add(mi);
   mi.setMnemonic(utils.getMnemonic(mnemonic));
   mi.getAccessibleContext().setAccessibleDescription(utils.getString(
                                                        accessibleDescription));
   mi.addActionListener(new ChangeThemeAction(this, theme));

   return mi;
  }
  
  /**
   * This method creates a generic JMenuItem to list internet links.
   *
   * @param menu    The menu to add the JMenuItem to
   * @param label   The components' s text
   * @param address The internet address of the link
   *
   * @return        A <code>JMenuItem</code> for the Links menu
   *
   * @since         0.1.3pre1
   */
  public JMenuItem createLinkMenuItem(JMenu menu, String label,
                                      final String address) {
    JMenuItem mi = (JMenuItem) menu.add(new JMenuItem(("<html><a href=>" + 
                                                       label + "</a></html>")));
    mi.setIcon(utils.createImageIcon16x16("Network.png"));
    mi.addActionListener(new LinkAction(this, address));

    return mi;
  }

  /**
   * This method creates JToggleButtons for the first tool bar.
   */
  public void createToggleButtons() {
    String[] togglebuttons = {"Exec", "RandomExec", "Settings", "Reload", "Exit",
                              "Help", "Contexthelp", "Java_src", "Mail", "Gohome"};

    Action[] action = {new PlayAction(this),
                       new RandomPlayAction(this),
                       new SettingsAction(this),
                       new UpdateListOfGamesAction(this),
                       new ExitAction(this),
		       null, null,
                       new SrcAction(this),
                       new MailAction(this),
                       new HomepageAction(this)};

    for (int i = 0; i < togglebuttons.length; i++) {
      JToggleButton tb = this.getToolBar().addToggleButton(action[i],
                                           togglebuttons[i],
					   utils.createImageIcon24x24(
                                           togglebuttons[i] + ".png"),
					   utils.getString(togglebuttons[i] +
                                           ".togglebutton_accessible_description"));
      this.getToolBarGroup().add(tb);
      if(this.getToolBarGroup().getSelection() == null) {
        tb.setSelected(true);
      }
      tb.setText(null);
      tb.setToolTipText(utils.getString(togglebuttons[i] +
                                       ".togglebutton_accessible_description"));

      if (i == 4)
        this.getToolBar().addSeparator();
    }
  }

  /**
   * This method creates a JToggleButton and some other components for the second tool bar.
   * With these components it' s possible to search a keyword in the list of 
   * games.
   */
  public void createToggleButtons2() {
    JLabel searchLabel = new JLabel();
    searchLabel.setMaximumSize(new Dimension(80, 20));
    searchLabel.setText(" " + utils.getString("SearchLabel.label"));
    this.getToolBar2().add(searchLabel);

    this.getToolBar2().addSeparator();

    // Combobox to select type of search: description or name
    String[] options = {" " + utils.getString("Table.column_description"),
                        " " + utils.getString("Table.column_name")};

    searchOptions = new JComboBox(options);
    searchOptions.setMaximumSize(new Dimension(120, 24));
    this.getToolBar2().add(searchOptions);

    this.getToolBar2().addSeparator();

    // A few options to make the search: ignore case option...
    ignoreCaseBox = new JRadioButton(utils.getString("Radio.ignore_case"));
    ignoreCaseBox.setMinimumSize(new Dimension(120, 20));
    ignoreCaseBox.setSelected(true);
    searchGroup.add(ignoreCaseBox);
    this.getToolBar2().add(ignoreCaseBox);

    // These are useful options: a lot of games have a name or a description
    // that starts or ends in the same way. Romeo will store all results in
    // a vector: the user will can highlight them pressing key F3.
    
    // The option to search game names that start with the keyword
    startsWithBox = new JRadioButton(utils.getString("Radio.starts_with") + " ");
    startsWithBox.setMinimumSize(new Dimension(120, 20));
    searchGroup.add(startsWithBox);
    this.getToolBar2().add(startsWithBox);

    // The option to search game names that end with the keyword
    endsWithBox = new JRadioButton(utils.getString("Radio.ends_with") + " ");
    endsWithBox.setMinimumSize(new Dimension(120, 20));
    searchGroup.add(endsWithBox);
    this.getToolBar2().add(endsWithBox);

    this.getToolBar2().addSeparator();

    // Search field
    searchField = new MemComboBox();
    searchField.setMaximumSize(new Dimension(600, 24));
    searchField.load(System.getProperty("user.home") + File.separator +
                     ".romeo4xmame" + File.separator + "queries.dat");
    this.getToolBar2().add(searchField);

    this.getToolBar2().addSeparator();

    // Button to start the search
    JToggleButton tb = this.getToolBar2().addToggleButton(new FindAction(this),
                                         "Find",
					 utils.createImageIcon24x24("Find.png"),
					 utils.getString(
                                   "Find.togglebutton_accessible_description"));
    this.getToolBarGroup2().add(tb);
    tb.setText(null);
    tb.setToolTipText(utils.getString(
                                   "Find.togglebutton_accessible_description"));
  }

  /**
   * This method creates the status bar.
   *
   * @return <code>JPanel</code>
   */
  public JPanel createStatusBar() {
    statusbarPanel = new JPanel();
    statusbarPanel.setLayout(new BorderLayout());

    statusField = new JTextField();
    statusField.setEditable(false);
    statusField.setPreferredSize(new Dimension(300, 20));
    // Get mame version
    xmame = new Xmame(this);
    String version = xmame.getVersion(
                               games.xmameComboBox.getSelectedItem().toString());
    statusField.setText(" " + version);
    statusbarPanel.add(statusField, BorderLayout.WEST);
    
    // Hello the user on startup :)
    String hello;
    String userName = config.get("User.name");
    // Hello the user
    if (!userName.equals(""))
      hello = utils.getString("Romeo.Hello") + " " + userName + "!";
    else
      hello = utils.getString("Romeo.Hello") + " " +
              System.getProperty("user.name") + "!";
    statusField2 = new JTextField(" " + hello);
    statusField2.setPreferredSize(new Dimension(250, 20));
    statusField2.setEditable(false);
    statusbarPanel.add(statusField2, BorderLayout.CENTER);
    
    statusPanel = new JPanel();
    statusPanel.setLayout(new BorderLayout());

    statusField3 = new JTextField();
    statusField3.setPreferredSize(new Dimension(125, 20));
    statusField3.setEditable(false);
    statusPanel.add(statusField3, BorderLayout.WEST);

    statusField4 = new JTextField();
    statusField4.setPreferredSize(new Dimension(125, 20));
    statusField4.setEditable(false);
    statusPanel.add(statusField4, BorderLayout.CENTER);

    progressBar = new JProgressBar();
    progressBar.setMinimum(0);
    progressBar.setStringPainted(true);
    statusPanel.add(progressBar, BorderLayout.EAST);

    statusbarPanel.add(statusPanel, BorderLayout.EAST);

    return statusbarPanel;
  }

  /**
   * This method brings up Romeo by showing the frame.
   *
   * @see #setStartupSkin()
   */
  public void showRomeo() {
    if(getFrame() != null) {
      // put Romeo in a frame and show it
      JFrame f = getFrame();
      f.setTitle(utils.getString("Romeo.Title"));
      Image icon = Toolkit.getDefaultToolkit().getImage(
                       getClass().getResource("/images/icons/16x16/Romeo.png"));
      f.setIconImage(icon);
      f.getContentPane().add(this, BorderLayout.CENTER);

      //******* Set size and location.
      SIZE_X = utils.getFrameParameter(getFrame(), ".SIZE_X");
      SIZE_Y = utils.getFrameParameter(getFrame(), ".SIZE_Y");
      f.setSize(SIZE_X, SIZE_Y);
      LOCATION_X = utils.getFrameParameter(getFrame(), ".LOCATION_X");
      LOCATION_Y = utils.getFrameParameter(getFrame(), ".LOCATION_Y");
      f.setLocation(LOCATION_X, LOCATION_Y);
      
      // Set a skin on startup
      setStartupSkin();

      // Show Romeo
      getFrame().show();
    }
  }
  
  /**
   * This method loads a skin on startup.
   *
   * @see #setSkinTheme(String)
   */
  public void setStartupSkin() {
    // Verify if the user wanna skin Romeo on startup
    if (config.getBoolean("Use_skin")) {
      // Get the themepack to load
      String themepack = config.get("Skin.path");
      // Call void 'setSkinTheme'
      setSkinTheme(themepack);
    }
  }

  // *******************************************************
  // ****************** Utility Methods ********************
  // *******************************************************

  /**
   * This method loads/unloads a skin.
   *
   * @param newSkin The path to the new skin to load
   * @param useSkin True if a skin has to be loaded, false if the
   *                the skin has to be unloaded
   *
   * @see #updateLookAndFeel()
   */
  public void updateSkinTheme(String newSkin, boolean useSkin) {
    if (!useSkin) {
      Skin skin = SkinLookAndFeel.getSkin();
      skin.unload();
      updateLookAndFeel();
      SwingUtilities.updateComponentTreeUI(this);
    }
    else
      setSkinTheme(newSkin);
  }

  /**
   * This method sets the skin.
   *
   * @param skin The skin to load
   */
  public void setSkinTheme(String skin) {
    try {
      if (skin.endsWith(".xml")) {
        SkinLookAndFeel.setSkin(SkinLookAndFeel.loadThemePackDefinition(
                                SkinUtils.toURL(new File(skin))));
      }
      else {
        SkinLookAndFeel.setSkin(SkinLookAndFeel.loadThemePack(skin));
      }
      SkinLookAndFeel.enable();

      Component c = SwingUtilities.getAncestorOfClass(java.awt.Window.class, this);
      if (c == null) {
        c = SwingUtilities.getAncestorOfClass(Romeo.class, this);
      }

      if (c != null) {
        SwingUtilities.updateComponentTreeUI(c);
      }

      // lazily update the UI's for the remaining modules
      Frame[] frame = getFrame().getFrames();

      for (int i = 0; i < frame.length; i++) {
        if (frame[i] != null)
          SwingUtilities.updateComponentTreeUI(frame[i]);
      }

    } catch (Exception e) {
      e.printStackTrace();
    } // end of try-catch
  }

  /**
   * An utility function that layers on top of the LookAndFeel's
   * isSupportedLookAndFeel() method. Returns true if the LookAndFeel
   * is supported. Returns false if the LookAndFeel is not supported
   * and/or if there is any kind of error checking if the LookAndFeel
   * is supported.
   *
   * The L&F menu will use this method to detemine whether the various
   * L&F options should be active or inactive.
   *
   * @param  laf The look and feel
   *
   * @return <code>True</code> if the LookAndFeel is supported;
   *         <code>false</code> otherwise.
   */
  protected boolean isAvailableLookAndFeel(String laf) {
    try {
      Class lnfClass = Class.forName(laf);
      LookAndFeel newLAF = (LookAndFeel)(lnfClass.newInstance());
      return newLAF.isSupportedLookAndFeel();
    }
    catch(Exception e) { // If ANYTHING weird happens, return false
      return false;
    }
  }

  /**
   * This method stores the current L&F, and calls updateLookAndFeel().
   *
   * @param laf The LnF
   *
   * @see       #updateLookAndFeel()
   */
  public void setLookAndFeel(String laf) {
    if(currentLookAndFeel != laf) {
      currentLookAndFeel = laf;
      themesMenu.setEnabled(laf == metal);
      updateLookAndFeel();
    }
  }

  /**
   * This method sets the current L&F on each frame.
   */
  public void updateLookAndFeel() {
    try {
      UIManager.setLookAndFeel(currentLookAndFeel);
      SwingUtilities.updateComponentTreeUI(this);

      // lazily update the UI's for the remaining modules
      Frame[] frame = getFrame().getFrames();

      for (int i = 0; i < frame.length; i++) {
        if (frame[i] != null)
          SwingUtilities.updateComponentTreeUI(frame[i]);
      }
    }
    catch (Exception ex) {
      System.out.println("Failed loading L&F: " + currentLookAndFeel);
      System.out.println(ex);
    }
  }

  /**
   * This method returns the frame instance.
   *
   * @return <code>JFrame</code>
   */
  public static JFrame getFrame() {
    return frame;
  }

  /**
   * This method returns the menubar.
   *
   * @return <code>JMenuBar</code>
   */
  public JMenuBar getJMenuBar() {
    return menuBar;
  }

  /**
   * This method returns the tool bar.
   *
   * @return <code>ToggleButtonToolBar</code>
   */
  public ToggleButtonToolBar getToolBar() {
    return toolbar;
  }

  /**
   * This method returns the tool bar button group.
   *
   * @return <code>ButtonGroup</code>
   */
  public ButtonGroup getToolBarGroup() {
    return toolbarGroup;
  }

  /**
   * This method returns the second tool bar.
   *
   * @return <code>ToggleButtonToolBar</code>
   */
  public ToggleButtonToolBar getToolBar2() {
    return toolbar2;
  }

  /**
   * This method returns the second tool bar button group.
   *
   * @return <code>ButtonGroup</code>
   */
  public ButtonGroup getToolBarGroup2() {
    return toolbarGroup2;
  }

  /**
   * This method returns the tabbed pane.
   *
   * @return <code>JTabbedPane</code>
   */
  public JTabbedPane getTabbedPane() {
    return tabbedPane;
  }

  /**
   * This method returns the content pane.
   *
   * @return <code>Container</code>
   */
  public Container getContentPane() {
    if(contentPane == null) {
      if(getFrame() != null) {
	contentPane = getFrame().getContentPane();
       }
     }
     return contentPane;
  }

  /**
   * This method creates a frame for Romeo to reside in.
   */
  public JFrame createFrame() {
    JFrame frame = new JFrame();

    // Create a window listener and add it to the frame
    WindowListener l = new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        utils.saveFrameParameters(getFrame(), info);
	searchField.save(System.getProperty("user.home") + File.separator + 
	                 ".romeo4xmame" + File.separator + "queries.dat");
	System.exit(0);
      }
      
      Frame helpFrame = null;
      public void windowDeactivated(WindowEvent e) {
        if (helpFrame != null)
          return;
        Frame[] frames = getFrame().getFrames(); // Only available in Java 2
        for (int k = 0; k < frames.length; k++) {
          if (!(frames[k] instanceof JFrame))
            continue;
          JFrame jf = (JFrame)frames[k];
          if (jf.getContentPane().getComponentCount() == 0)
            continue;
          Component c = jf.getContentPane().getComponent(0);
          if (c == null || !(c instanceof JHelp))
            continue;
          helpFrame = jf;
          JHelp jh = (JHelp)c;
          for (int s = 0; s < jh.getComponentCount(); s++) {
            c = jh.getComponent(s);
            if (c == null || !(c instanceof JToolBar))
              continue;
            JToolBar jtb = (JToolBar)c;
            jtb.addSeparator();
            JButton home = new JButton(utils.createImageIcon24x24("Gohome.png"));
            home.setToolTipText("Home Topic");
            jtb.add(home);
                       
            ActionListener alst = new ActionListener() {
              public void actionPerformed(ActionEvent e) {
                try {
                  helpBroker.setCurrentID(helpSet.getHomeID());
                }
                catch (Exception ex) {}
              }
            };
            home.addActionListener(alst);
          }
          Enumeration en = jh.getHelpNavigators();
          while (en.hasMoreElements()) {
            JHelpNavigator jhn = (JHelpNavigator) en.nextElement();
            NavigatorView view = jhn.getNavigatorView();
            if (view instanceof HistoryView)
              ((HistoryView)view).setNavigator(jhn);
          }
        }
      }
    };
    frame.addWindowListener(l);
    
    // Create a focus manager
    dfocus = new DefaultFocusManager();
    // Set as the current focus manager
    KeyboardFocusManager.setCurrentKeyboardFocusManager(dfocus);
    // Add a keyboard stroke interceptor to listen for F3 key pressed
    // Needed to highlight results of a search
    dfocus.addKeyEventDispatcher(this);

    return frame;
  }

  /**
   * This method locks/unlocks all container' s components.
   *
   * @param locked If <code>true</code> unlocks all components;
   *               else locks them.
   * @param c      The container containing the components to lock/unlock
   */
  public void setLocked(boolean locked, Container c) {
    Component[] comp = c.getComponents();
    for (int i = 0; i < comp.length; i++) {
      if (comp[i] instanceof Container) {
        // recursive call
        setLocked(locked, (Container)comp[i]);
      }
    }
    if (c instanceof JPanel) {
      if (locked) {
        c.setCursor(waitCursor);
      } else {
        c.setCursor(defaultCursor);
      }
    } else {
      c.setEnabled(!locked);
    }
  }

  // *******************************************************
  // **************   ToggleButtonToolbar  *****************
  // *******************************************************

  static Insets zeroInsets = new Insets(1,1,1,1);
  /**
   * This class extends JToolBar to create toggle buttons.
   */
  protected class ToggleButtonToolBar extends JToolBar {
    public ToggleButtonToolBar() {
      super();
    }

    JToggleButton addToggleButton(Action a, String tbName,
                                  ImageIcon tbIcon, String tbToolTipText) {
      JToggleButton tb = new JToggleButton(tbName, tbIcon);
      tb.setMinimumSize(new Dimension(30, 30));
      tb.setPreferredSize(new Dimension(30, 30));
      tb.setMaximumSize(new Dimension(30, 30));
      tb.setMargin(zeroInsets);
      tb.setText(null);
      tb.setEnabled(true);
      tb.setToolTipText(utils.getString(tbToolTipText));
      if (tbName.equals("Help"))
        helpBroker.enableHelpOnButton(tb, "top", null);
      else if (tbName.equals("Contexthelp"))
        tb.addActionListener(new CSH.DisplayHelpAfterTracking(helpBroker));
      else
        tb.setAction(a);
      tb.setIcon(tbIcon);
      add(tb);
      return tb;
    }
  }

  /**
   * This method is called by the current KeyboardFocusManager requesting 
   * that this KeyEventDispatcher dispatch the specified event on its behalf.
   * This KeyEventDispatcher is free to retarget the event, consume it, 
   * dispatch it itself, or make other changes. This capability is typically 
   * used to deliver KeyEvents to Components other than the focus owner. This 
   * can be useful when navigating children of non-focusable Windows in an 
   * accessible environment, for example. Note that if a KeyEventDispatcher 
   * dispatches the KeyEvent itself, it must use redispatchEvent  to prevent 
   * the current KeyboardFocusManager from recursively requesting that this 
   * KeyEventDispatcher dispatch the event again.
   * If an implementation of this method returns false, then the KeyEvent is 
   * passed to the next KeyEventDispatcher in the chain, ending with the current
   * KeyboardFocusManager. If an implementation returns true, the KeyEvent is 
   * assumed to have been dispatched (although this need not be the case), and 
   * the current KeyboardFocusManager will take no further action with regard 
   * to the KeyEvent. In such a case, KeyboardFocusManager.dispatchEvent should 
   * return true as well. If an implementation consumes the KeyEvent, but returns
   * false, the consumed event will still be passed to the next KeyEventDispatcher 
   * in the chain. It is important for developers to check whether the KeyEvent 
   * has been consumed before dispatching it to a target. By default, the current 
   * KeyboardFocusManager will not dispatch a consumed KeyEvent.
   * The addition of the KeyEventDispatcher class and the focus manager
   * allows the Java 1.4 program to work the same way it did in its
   * Java 1.3 version.
   *
   * @param  e The KeyEvent to dispatch
   *
   * @return   <code>true</code> if the key event has been consumed by
   *           this method and <code>false</code> if it has not.
   *
   * @see      com.miami.romeo.main.Games#highlightRow(JTable, int, int)
   * @see      #search(String)
   *
   * @since    0.1.3pre1
   */
  public boolean dispatchKeyEvent(KeyEvent e) {
    // Make sure the event is not consumed
    boolean consumed = e.isConsumed();
    // Take a reference to a JComboBox component and store it in an instance 
    // variable along with the JTextField component used as that combo box' s 
    // editor.
    JTextField m_editor = (JTextField)searchField.getEditor().getEditorComponent();
    // Receive pressed events
    if (e.getID() == KeyEvent.KEY_PRESSED) {
      // On the F3 key highlight a game in the list of games
      if (e.getKeyCode() == KeyEvent.VK_F3 && gamesTabIsSelected() && 
          !consumed) {
        if (searchResults != null && searchResults.size() > 1) {
	  value++;
	  if (value < searchResults.size()) {
	    int row = Integer.parseInt(searchResults.elementAt(
	                               value).toString().substring(4).trim());
	    games.highlightRow(games.table, row, 2);
	  }
          else {
	    int result = JOptionPane.showConfirmDialog(null,
                                                     utils.getString("Find.highlight"));

            if(result == JOptionPane.YES_OPTION) {
              value = 0;
	      int row = Integer.parseInt(searchResults.elementAt(
	                                 value).toString().substring(4).trim());
	      games.highlightRow(games.table, row, 2);
            }
            else if(result == JOptionPane.NO_OPTION) {
              /* Do nothing */
            }
	  }
	}

	consumed = true;
      }
      // On the ENTER key search the keyword
      else if (e.getKeyCode() == KeyEvent.VK_ENTER && 
               searchFieldHasFocus(m_editor) &&
               !consumed) {
	String keyword = m_editor.getText();
	if (keyword.length() != 0) {
	  search(keyword);
	}

	consumed = true;
      }
    }
    // Receive released events
    else if (e.getID() == KeyEvent.KEY_RELEASED) {
      // this method retrieves the pressed characters and verifies that 
      // they are not control characters. We also retrieve the contents 
      // of the text field and check that it is not empty (to avoid annoying 
      // the user with suggestions in an empty field).  
      if (searchFieldHasFocus(m_editor) && !consumed) {
        char ch = e.getKeyChar();
        if (ch != KeyEvent.CHAR_UNDEFINED && !Character.isISOControl(ch)) {
          int pos = m_editor.getCaretPosition();
          String str = m_editor.getText();
          if (str.length() != 0) {
            for (int k = 0; k < searchField.getItemCount(); k++) {
              String item = searchField.getItemAt(k).toString();
              if (item.startsWith(str)) {
	        m_editor.setText(item);
	        m_editor.setCaretPosition(item.length());
	        m_editor.moveCaretPosition(pos);
	        break;
	      }
	    }
          }
	}
      }
      
      consumed = true;
    }

    return consumed; // true means not passed on
  }
  

  /**
   * This method returns true if the first tab of the JtabbedPane
   * is selected, false otherwise.
   *
   * @return <code>True</code> if the first tab of the JtabbedPane
   *         is selected, <code>false</code> otherwise.
   */
  public boolean gamesTabIsSelected() {
    int index = tabbedPane.getSelectedIndex();
    
    if (index == 0)
      return true;
    else
      return false;
  }

  /**
   * This method returns true if the search field has focus,
   * false otherwise.
   *
   * @param  m_editor The combo box' s editor
   *
   * @return <code>True</code> if the search field has focus,
   *         <code>false</code> otherwise.
   */
  public boolean searchFieldHasFocus(JTextField m_editor) {
    Component comp = dfocus.getFocusOwner();
    
    if (comp == m_editor)
      return true;
    else
      return false;
  }
  /*
  public void mouseClicked(MouseEvent e) {}
        
  public void mouseEntered(MouseEvent e) {
    Image im = Toolkit.getDefaultToolkit().getImage(
                   getClass().getResource("images/cursors/default_cursor.jpg"));
    Cursor customCursor = Toolkit.getDefaultToolkit().createCustomCursor(
                                          im, new Point(0, 0), "Custom Cursor");
    setCursor(customCursor);
  }

  public void mouseExited(MouseEvent e) {}
  
  public void mousePressed(MouseEvent e) {}

  public void mouseReleased(MouseEvent e) {}
  */
  
  /**
   * This method adds the context sensitive help.
   *
   * @since 0.1.3pre1
   */
  protected void addContextHelp() {
    // Add a context sensitive help to some components
    CSH.setHelpIDString(getToolBar().getComponentAtIndex(0), "Play");
    CSH.setHelpIDString(getToolBar().getComponentAtIndex(1), "Settings");
    CSH.setHelpIDString(getToolBar().getComponentAtIndex(2), "Lists");
    CSH.setHelpIDString(getToolBar().getComponentAtIndex(3), "Exiting");
    CSH.setHelpIDString(getToolBar().getComponentAtIndex(7), "Src");
    CSH.setHelpIDString(getToolBar().getComponentAtIndex(8), "Mail");
    CSH.setHelpIDString(getToolBar().getComponentAtIndex(9), "Home");
    CSH.setHelpIDString(searchOptions, "Search");
    CSH.setHelpIDString(searchField, "Search");
    CSH.setHelpIDString(ignoreCaseBox, "Search");
    CSH.setHelpIDString(startsWithBox, "Search");
    CSH.setHelpIDString(endsWithBox, "Search");
    CSH.setHelpIDString(games.xmameComboBox, "Multiple");
    CSH.setHelpIDString(games.tree, "Tree");
    CSH.setHelpIDString(games.table2, "Table");
    CSH.setHelpIDString(games.table, "Table");
    CSH.setHelpIDString(games.table3, "Table");
    CSH.setHelpIDString(info.desktop, "Desktop");
  }

  // *******************************************************
  // ********************   Actions  ***********************
  // *******************************************************

  /**
   * This class runs a selected game.
   */
  class PlayAction extends AbstractAction {
    Romeo romeo;
    protected PlayAction(Romeo romeo) {
      super("PlayAction");
      this.romeo = romeo;
    }

    public void actionPerformed(ActionEvent e) {
      options.runMame();
    }
  }

  /**
   * This class runs a random game.
   */
  class RandomPlayAction extends AbstractAction {
    Romeo romeo;
    protected RandomPlayAction(Romeo romeo) {
      super("RandomPlayAction");
      this.romeo = romeo;
    }

    public void actionPerformed(ActionEvent e) {
      DataIO dataio = new DataIO(romeo);
      String xmamePath = games.getXmamePath();
      int count = dataio.countRoms(xmamePath);
      if (count > 0) {
        int sum = 0;
        Random rand = new Random();
        sum += rand.nextInt(count);
        Vector aRom = dataio.getAvailableRoms();
        Object[] data = (Object[])aRom.elementAt(sum);
        String gameName = (String)data[0];
        String[] romDir = games.getRomDirs();
        String romPath = " -rompath " + games.getGamePath(romDir,
                                                          gameName.substring(0,
                                                          gameName.indexOf(".zip")));
        try {
          config = new Config(new File(System.getProperty("user.home") +
                                       File.separator + ".romeo4xmame" +
				       File.separator + "Romeo.ini"));

          int n = config.getInt("Total_of_played_games");
          int MAX = n + 1;
          config.add("Total_of_played_games", MAX);
          config.save();

          games.setNumberOfPlayedMatches(gameName, MAX);

          boolean iconified = config.getBoolean("Iconify_before_playing");

          if (iconified) {
            Frame[] frame = getFrame().getFrames();

            for (int i = 0; i < frame.length; i++) {
              if (frame[i] != null)
                frame[i].setState(Frame.ICONIFIED);
            }
          }

          xmame = new Xmame(romeo);
          xmame.runMame(xmamePath, " " + gameName + romPath);
        }
        catch (IOException ioe) {
          System.err.println(ioe.getMessage());
        }
      }
    }
  }

  /**
   * This class runs Settings window.
   */
  class SettingsAction extends AbstractAction {
    Romeo romeo;
    protected SettingsAction(Romeo romeo) {
      super("SettingsAction");
      this.romeo = romeo;
    }

    public void actionPerformed(ActionEvent e) {
      if (settings == null)
        settings = new Settings(Romeo.this);
      else
        settings.setVisible(true);
    }
  }

  /**
   * This class updates the list of games.
   */
  class UpdateListOfGamesAction extends AbstractAction {
    Romeo romeo;
    protected UpdateListOfGamesAction(Romeo romeo) {
      super("UpdateListOfGamesAction");
      this.romeo = romeo;
    }

    public void actionPerformed(ActionEvent e) {
      if (updateDialog  == null || !updateDialog.isVisible()) {
        games.saveListOfGames();
      }
    }
  }

  /**
   * This class closes the application.
   */
  class ExitAction extends AbstractAction {
    Romeo romeo;
    protected ExitAction(Romeo romeo) {
      super("ExitAction");
      this.romeo = romeo;
    }

    public void actionPerformed(ActionEvent e) {
      utils.saveFrameParameters(getFrame(), info);
      searchField.save(System.getProperty("user.home") + File.separator + 
                       ".romeo4xmame" + File.separator + "queries.dat");
      System.exit(0);
    }
  }

  /**
   * This class shows/hides the tool bars.
   */
  class ViewToolBarAction extends AbstractAction {
    Romeo romeo;
    String toolBar;
    protected ViewToolBarAction(Romeo romeo, String toolBar) {
      super("ViewToolBarAction");
      this.romeo = romeo;
      this.toolBar = toolBar;
    }

    public void actionPerformed(ActionEvent e) {
      JCheckBoxMenuItem mi = (JCheckBoxMenuItem) e.getSource();

      if (toolBar.equals("first")) {
        if (mi.isSelected()) {
          toolbarPanel.add(toolbar, BorderLayout.NORTH);

	  SwingUtilities.updateComponentTreeUI(Romeo.this);

	  config.add("View.toolbar1", true);
	}
	else {
	  toolbarPanel.remove(toolbar);

          SwingUtilities.updateComponentTreeUI(Romeo.this);

	  config.add("View.toolbar1", false);
	}
      }
      else {
        if (mi.isSelected()) {
	  toolbarPanel.add(toolbar2, BorderLayout.SOUTH);

	  SwingUtilities.updateComponentTreeUI(Romeo.this);

	  config.add("View.toolbar2", true);
	}
	else {
	  toolbarPanel.remove(toolbar2);

	  SwingUtilities.updateComponentTreeUI(Romeo.this);

	  config.add("View.toolbar2", false);
	}
      }

      config.save();
    }
  }
  
  /**
   * This class shows/hides the message area.
   */
  class ViewMessageAreaAction  extends AbstractAction {
    Romeo romeo;
    protected ViewMessageAreaAction(Romeo romeo) {
      super("ViewMessageAreaAction");
      this.romeo = romeo;
    }

    public void actionPerformed(ActionEvent e) {
      JCheckBoxMenuItem mi = (JCheckBoxMenuItem) e.getSource();

      if (mi.isSelected()) {
        games.vertSplitPane.setRightComponent(games.messageAreaPanel);

	SwingUtilities.updateComponentTreeUI(Romeo.this);

	config.add("View.messageArea", true);
      }
      else if (!mi.isSelected()) {
        games.vertSplitPane.remove(1);

	SwingUtilities.updateComponentTreeUI(Romeo.this);

	config.add("View.messageArea", false);
      }

      config.save();
    }
  }

  /**
   * This class shows/hides the status bar.
   */
  class ViewStatusBarAction extends AbstractAction {
    Romeo romeo;
    protected ViewStatusBarAction(Romeo romeo) {
      super("ViewStatusBarAction");
      this.romeo = romeo;
    }

    public void actionPerformed(ActionEvent e) {
      JCheckBoxMenuItem mi = (JCheckBoxMenuItem) e.getSource();

      if (mi.isSelected()) {
	statusbarPanel.add(statusField, BorderLayout.WEST);
        statusbarPanel.add(statusField2, BorderLayout.CENTER);
	statusbarPanel.add(statusPanel, BorderLayout.EAST);

	SwingUtilities.updateComponentTreeUI(Romeo.this);

	config.add("View.statusbar", true);
      }
      else if (!mi.isSelected()) {
        statusbarPanel.removeAll();

	SwingUtilities.updateComponentTreeUI(Romeo.this);

	config.add("View.statusbar", false);
      }

      config.save();
    }
  }

  /**
   * This class changes the theme.
   */
  class ChangeThemeAction extends AbstractAction {
    Romeo romeo;
    DefaultMetalTheme theme;
    protected ChangeThemeAction(Romeo romeo, DefaultMetalTheme theme) {
      super("ChangeTheme");
      this.romeo = romeo;
      this.theme = theme;
    }

    public void actionPerformed(ActionEvent e) {
      MetalLookAndFeel.setCurrentTheme(theme);
      romeo.updateLookAndFeel();
    }
  }

  /**
   * This class changes the look and feel.
   */
  class ChangeLookAndFeelAction extends AbstractAction {
    Romeo romeo;
    String laf;
    protected ChangeLookAndFeelAction(Romeo romeo, String laf) {
      super("ChangeTheme");
      this.romeo = romeo;
      this.laf = laf;
    }

    public void actionPerformed(ActionEvent e) {
      romeo.setLookAndFeel(laf);
    }
  }
  
  /**
   * This class runs an integrated html browser or the System
   * one set by the user and redirects it to a given web site.
   */
  class LinkAction extends AbstractAction {
    Romeo romeo;
    String address;
    protected LinkAction(Romeo romeo, String address) {
      super("LinkAction");
      this.romeo = romeo;
      this.address = address;
    }

    public void actionPerformed(ActionEvent e) {
      Thread thread = new Thread() {
        public void run() {
          utils.goToUrl(address);
	}
      };
      thread.start();
    }
  }

  /**
   * This class runs a viewer to read the source code.
   */
  class SrcAction extends AbstractAction {
    Romeo romeo;
    protected SrcAction(Romeo romeo) {
      super("SrcAction");
      this.romeo = romeo;
    }

    public void actionPerformed(ActionEvent e) {
      if(src == null)
	src = new Src();
      else
        src.setVisible(true);
    }
  }

  /**
   * This class runs an integrated html browser or the System
   * one set by the user and redirects it to Romeo' s home page.
   */
  class HomepageAction extends AbstractAction {
    Romeo romeo;
    protected HomepageAction(Romeo romeo) {
      super("HomepageAction");
      this.romeo = romeo;
    }

    public void actionPerformed(ActionEvent e) {
      final String homepage = "http://stefanobolli.sourceforge.net/";

      Thread thread = new Thread() {
        public void run() {
          utils.goToUrl(homepage);
	}
      };
      thread.start();
    }
  }

  /**
   * This class verifies if a new version of Romeo has been
   * released.
   */
  class ReleaseAction extends AbstractAction {
    Romeo romeo;
    protected ReleaseAction(Romeo romeo) {
      super("ReleaseAction");
      this.romeo = romeo;
    }

    public void actionPerformed(ActionEvent e) {
      checkNewRelease();
    }
  }

  /**
   * This class runs a mail module to send e-mail to
   * the author.
   */
  class MailAction extends AbstractAction {
    Romeo romeo;
    protected MailAction(Romeo romeo) {
      super("MailAction");
      this.romeo = romeo;
    }

    public void actionPerformed(ActionEvent e) {
      if(mailer == null)
	mailer = new Mailer();
      else
        mailer.setVisible(true);
    }
  }

  /**
   * This class searchs for a game in the list of games.
   */
  class FindAction extends AbstractAction {
    Romeo romeo;
    protected FindAction(Romeo romeo) {
      super("FindAction");
      this.romeo = romeo;
    }

    public void actionPerformed(ActionEvent e) {
      String keyword = searchField.getSelectedItem().toString();
      if (keyword.length() > 0)
        search(keyword);
    }
  }

  /**
   * This class runs the About box.
   */
  class AboutAction extends AbstractAction {
    Romeo romeo;
    protected AboutAction(Romeo romeo) {
      super("AboutAction");
      this.romeo = romeo;
    }

    public void actionPerformed(ActionEvent e) {
      if(aboutBox == null) {
	// JPanel panel = new JPanel(new BorderLayout());
	JPanel panel = new AboutPanel(romeo);

	aboutBox = new JDialog(getFrame(), utils.getString("AboutBox.title"),
                               false);
	aboutBox.getContentPane().add(panel, BorderLayout.CENTER);

        // JButton button = new JButton(utils.getString(
	//                                         "AboutBox.ok_button_text"));
	JPanel buttonpanel = new JPanel();
	buttonpanel.setOpaque(false);
	JButton button = (JButton) buttonpanel.add(
          new JButton(utils.getString("AboutBox.ok_button_text"))
	);
	panel.add(buttonpanel, BorderLayout.SOUTH);
	button.addActionListener(new OkAction(aboutBox));
      }

      aboutBox.setSize(400, 300);
      aboutBox.setResizable(false);
      //Point p = romeo.getLocationOnScreen();
      //aboutBox.setLocation(p.x + 10, p.y +10);
      Dimension screenDim = Toolkit.getDefaultToolkit().getScreenSize();
      Rectangle frameDim = aboutBox.getBounds();
      aboutBox.setLocation((screenDim.width  - frameDim.width)  / 2,
                           (screenDim.height - frameDim.height) / 2);
			   
      //  Handle escape key to close the dialog
      KeyStroke escapeKeyStroke = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0, false);
      Action escapeAction = new AbstractAction(){
        public void actionPerformed(ActionEvent e) {
	  scrollText.stop();
          aboutBox.setVisible(false);
	}
      };
      aboutBox.getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(
                                                     escapeKeyStroke, "ESCAPE");
      aboutBox.getRootPane().getActionMap().put("ESCAPE", escapeAction);

      scrollText.start();
      mm.surf.start();

      aboutBox.show();
    }
  }

  /**
   * This class extends JPanel to create a container for the
   * About Box.
   */
  class AboutPanel extends JPanel {
    Romeo romeo = null;
    AboutRomeo ab = null;
    SystemInfo systemInfo = null;
    License license = null;
    JTabbedPane aboutTabbedPane = null;

    public AboutPanel(Romeo romeo) {
      this.romeo = romeo;
      setLayout(new BorderLayout());
      setOpaque(false);

      aboutTabbedPane = new JTabbedPane();
      ab = new AboutRomeo();
      aboutTabbedPane.addTab(utils.getString("AboutBox.title"),
                             utils.createImageIcon16x16("Info.png"), ab);
      scrollText = new ScrollText();
      aboutTabbedPane.addTab(utils.getString("AboutBox.credits_label"),
                             utils.createImageIcon16x16("Video.png"),
                             scrollText);
      systemInfo = new SystemInfo();
      mm = new MemoryMonitor();
      systemInfo.sPane.addTab("Memory Monitor", null, mm);
      aboutTabbedPane.addTab(utils.getString("AboutBox.system_label"),
                             utils.createImageIcon16x16("Computer.gif"),
                             systemInfo);
      license = new License();
      aboutTabbedPane.addTab(utils.getString("AboutBox.license_label"),
                             utils.createImageIcon16x16("Text.png"),
                             license);
      add(aboutTabbedPane, BorderLayout.CENTER);
    }
  }

  /**
   * This class closes the About box
   */
  class OkAction extends AbstractAction {
    JDialog aboutBox;

    protected OkAction(JDialog aboutBox) {
      super("OkAction");
      this.aboutBox = aboutBox;
    }

    public void actionPerformed(ActionEvent e) {
      scrollText.stop();
      mm.surf.stop();
      aboutBox.setVisible(false);
    }
  }

  // *******************************************************
  // **********************  Misc  *************************
  // *******************************************************

  /**
   * This method verifies if a new version of Romeo has been
   * released.
   */
  public void checkNewRelease() {
    // human readable product version
    String romeoVersion = "Romeo-0.1.3pre4-binaries";
    //
    String destination = System.getProperty("user.home");
    String release;
    String filename;

    try {
      // File containing latest product version info
      URL url = new URL(
                  "http://stefanobolli.sourceforge.net/files/release.txt");

      URLConnection con = url.openConnection();

      BufferedReader b = new BufferedReader(new InputStreamReader(
                                                         con.getInputStream()));

      do {
        // Read the file
	release = b.readLine();
        if (release.startsWith("Latest available release is:")){
          int suffix = release.indexOf(".tar.bz2");
	  filename = release.substring(28, suffix).trim();
          //System.out.println(filename);

	  // A new release is available
	  if (!filename.equals(romeoVersion)) {
            // Ask if user wanna download it
	    int result = JOptionPane.showConfirmDialog(null,
                               utils.getString(
                               "OptionPane.new_release_confirmquestion"));

            if(result == JOptionPane.YES_OPTION) {
              //Create a file chooser
              JFileChooser fc = new JFileChooser();
	      fc.setSelectedFile(new File(filename + ".zip"));
              int returnVal = fc.showSaveDialog(Romeo.this);
              if (returnVal == JFileChooser.APPROVE_OPTION) {
                String path = fc.getSelectedFile().getPath();
	        int idx = path.lastIndexOf(File.separator);
	        destination = path.substring(0, idx + 1);
	        downloader = new Downloader(
		 // Download from a SourceForge mirror
		 "http://easynews.dl.sourceforge.net/sourceforge/stefanobolli/",
                 destination, filename + ".zip");
              }
            }
            else if(result == JOptionPane.NO_OPTION) {
              JOptionPane.showMessageDialog(null, utils.getString(
                                            "Messagebox.update_not_confirmed"),
                                            utils.getString(
                                            "Messagebox.info_title"),
                                            JOptionPane.INFORMATION_MESSAGE);
            }
          }
          else {
            JOptionPane.showMessageDialog(null, utils.getString(
                                          "Messagebox.update_not_available"),
                                          utils.getString(
                                          "Messagebox.info_title"),
                                          JOptionPane.INFORMATION_MESSAGE);
          }
        }

	break;
      } while (release != null);

      b.close();
    }
    catch(java.net.MalformedURLException mue) {
      System.err.println(mue.getMessage());
    }
    catch(IOException ioe) {
      System.err.println(ioe.getMessage());
    }
  }

  /**
   * This method searchs for a description or name in the list of games.
   *
   * @param keyword The keyword to search
   */
  public void search(String keyword) {
    if (games.model.getRowCount() > 0) {
      searchField.add(keyword);
      searchResults = new Vector();

      if (value != 0)
        value = 0;
	 
      int result = 0;

      // Search for a description
      if (searchOptions.getSelectedItem().toString().equals(" " +
                                         utils.getString("Table.column_description"))) {
	// Perform a search ignoring case
	if (ignoreCaseBox.isSelected()) {
          for (int i = 0; i < games.table.getRowCount(); i++) {
            if (games.table.getValueAt(i, 2).toString().equalsIgnoreCase(keyword)) {
	      result = 1;
	      games.highlightRow(games.table, i, 2);
	      break;
            }
          }
	}
	// Retrieve all descriptions that start with a given keyword
	else if (startsWithBox.isSelected()) {
          for (int i = 0; i < games.table.getRowCount(); i++) {
            if (games.table.getValueAt(i, 2).toString().startsWith(keyword)) {
              result = 1;
	      String searchResult = "Row " + i;
	      searchResults.addElement(searchResult);
            }
          }

	  if (result == 1) {
	    int row = Integer.parseInt(searchResults.elementAt(
	                               0).toString().substring(4).trim());
	    games.highlightRow(games.table, row, 2);

	    if (searchResults.size() > 0)
	      setStatusFieldMessage();
	  }
	}
	// Retrieve all descriptions that end with a given keyword
	else if (endsWithBox.isSelected()) {
          for (int i = 0; i < games.table.getRowCount(); i++) {
            if (games.table.getValueAt(i, 2).toString().endsWith(keyword)) {
              result = 1;
	      String searchResult = "Row " + i;
	      searchResults.addElement(searchResult);
            }
          }

	  if (result == 1) {
	    int row = Integer.parseInt(searchResults.elementAt(
	                               0).toString().substring(4).trim());
	    games.highlightRow(games.table, row, 2);

	    if (searchResults.size() > 0)
	      setStatusFieldMessage();
	  }
	}
      }
      // Search for a game name
      else {
        // Perform a search ignoring case
        if (ignoreCaseBox.isSelected()) {
          for (int i = 0; i < games.model.getRowCount(); i++) {
            if (games.table2.getValueAt(i, 0).toString().equalsIgnoreCase(keyword)) {
              result = 1;
	      games.highlightRow(games.table2, i, 0);
	      break;
            }
	  }
        }
	// Retrieve all names that starts with a given keyword
	else if (startsWithBox.isSelected()) {
          for (int i = 0; i < games.model.getRowCount(); i++) {
            if (games.table2.getValueAt(i, 0).toString().startsWith(keyword)) {
              result = 1;
	      String searchResult = "Row " + i;
	      searchResults.addElement(searchResult);
            }
	  }

	  if (result == 1) {
	    int row = Integer.parseInt(searchResults.elementAt(
	                               0).toString().substring(4).trim());
	    games.highlightRow(games.table2, row, 0);

	    if (searchResults.size() > 0)
	      setStatusFieldMessage();
	  }
        }
	// Retrieve all names that end with a given keyword
	else if (endsWithBox.isSelected()) {
          for (int i = 0; i < games.model.getRowCount(); i++) {
            if (games.table2.getValueAt(i, 0).toString().endsWith(keyword)) {
              result = 1;
	      String searchResult = "Row " + i;
	      searchResults.addElement(searchResult);
            }
	  }

	  if (result == 1) {
	    int row = Integer.parseInt(searchResults.elementAt(
	                               0).toString().substring(4).trim());
	    games.highlightRow(games.table2, row, 0);

	    if (searchResults.size() > 0)
	      setStatusFieldMessage();
	  }
        }
      }

      if (result == 0)
        JOptionPane.showMessageDialog(null, utils.getString(
                                      "Messagebox.error_stringnotfound"),
                                      utils.getString("Messagebox.error_title"),
                                      JOptionPane.ERROR_MESSAGE);
    }
    else {
      JOptionPane.showMessageDialog(null, utils.getString(
                                    "Messagebox.error_search"),
                                    utils.getString("Messagebox.error_title"),
                                    JOptionPane.ERROR_MESSAGE);
    }
  }

  /**
   * This method tells the user to press key F3 to highlight results
   * found by void search().
   *
   * @see   #search(String)
   *
   * @since 0.1.3pre1
   */
  public void setStatusFieldMessage() {
    if (searchResults.size() > 1) {
      // Save the previous message
      final String previousMessage = statusField2.getText();
      
      // Save previous foreground color
      final Color c = statusField2.getForeground();

      String foundGames = utils.getString("Find.results");

      int idx_$ = foundGames.indexOf('$');

      final String[] messages = {foundGames.substring(0, idx_$) +
	                         searchResults.size() +
			         foundGames.substring(idx_$ + 1),
			         " "};

      games.appendMessage("Info", messages[0]);

      if (timer != null) {
        timer.stop();
	timer = null;
      }

      //Create a timer to start the animation.
      timer = new javax.swing.Timer(1000, new ActionListener() {
        public void actionPerformed(ActionEvent evt) {
	  statusField2.setForeground(Color.red);
          statusField2.setText(messages[message]);
	  message++;
          if (message == 2) {
            message = 0;
            timer.restart();
	    loop++;
	    if (loop == 5) {
	      timer.stop();
	      statusField2.setForeground(c);
              statusField2.setText(previousMessage);
	      loop = 0;
	    }
          }
        }
      });

      timer.start();
    }
  }

  /**
   * Before the J2SE 1.4 release, all Swing components have included the advisory
   * that "A future release of Swing will provide support for long term persistence."
   * This has been replaced with the notice that "As of 1.4, support for long term
   * storage of all JavaBeans has been added to the java.beans package."
   * The XMLEncoder class in the java.beans package allows us to persist a JavaBean
   * in an XML file. Among other things, this provides a way to save Swing components.
   * After we save a Swing component, we can reconstitute the component with the XMLDecoder
   * class in the the java.beans package.
   *
   * this method instantiates an XMLEncoder object, and then uses it to store an XML
   * representation of the JFrame in the file Romeo.xml.
   *
   * @throws <code>Exception</code> DOCUMENT ME!
   *
   * @since  0.1.3pre2
   */
  protected void saveComponents() throws Exception {
    XMLEncoder encoder = new XMLEncoder(new BufferedOutputStream(
                                        new FileOutputStream(System.getProperty("user.home") +
                                                             File.separator + ".romeo4xmame" +
				                             File.separator + "Romeo.xml")));
    encoder.writeObject(getFrame());
    encoder.close();
  }
}

/**
 * This class extends JComboBox to add a historical mechanism for this
 * component. The constructor creates an underlying JComboBox component 
 * and sets its editable property to true. The component created by this
 * class permits to search a game for description or name in the list of
 * games.
 */
class MemComboBox  extends JComboBox {
  public static final int MAX_MEM_LEN = 30;

  /**
   * The constructor creates an editable combo box.
   */
  public MemComboBox() {
    super();
    setEditable(true);
  }

  /**
   * The add() method adds a new text string to the beginning of the list.
   * If this item is already present in the list, it is removed from the
   * old position. If the resulting list is longer than the pre-defined
   * maximum length then the last item in the list is truncated.
   *
   * @param item The text string to add to the list
   */
  public void add(String item) {
    removeItem(item);
    insertItemAt(item, 0);
    setSelectedItem(item);
    if (getItemCount() > MAX_MEM_LEN)
      removeItemAt(getItemCount()-1);
  }

  /**
   * Method load() loads a previously stored ComboBoxModel from file ?addresses.dat?
   * using the serialization mechanism. The significant portion of this method reads
   * an object from an ObjectInputStream and sets it as the ComboBoxModel. Note that
   * any possible exceptions are only printed to the standard output and purposefully
   * do not distract the user (since this serialization mechanism should be considered
   * an optional feature).
   *
   * @param fName The previously stored ComboBoxModel
   */
  public void load(String fName) {
    try {
      if (getItemCount() > 0)
        removeAllItems();

      File f = new File(fName);

      if (!f.exists())
        return;

      FileInputStream fStream = new FileInputStream(f);
      ObjectInput stream = new ObjectInputStream(fStream);

      Object obj = stream.readObject();

      if (obj instanceof ComboBoxModel)
        setModel((ComboBoxModel)obj);

      stream.close();
      fStream.close();
    }
    catch (Exception e) {
      e.printStackTrace();
      System.err.println("Serialization error: " + e.toString());
    }
  }

  /**
   * Similarly, the save() method serializes our combo box' s ComboBoxModel.
   * Any possible exceptions are, again, printed to standard output and do
   * not distract the user.
   *
   * @param fName Combo box' s ComboBoxModel
   */
  public void save(String fName) {
    try {
      FileOutputStream fStream = new FileOutputStream(fName);
      ObjectOutput stream = new ObjectOutputStream(fStream);

      stream.writeObject(getModel());

      stream.flush();
      stream.close();
      fStream.close();
    }
    catch (Exception e) {
      e.printStackTrace();
      System.err.println("Serialization error: " + e.toString());
    }
  }
} // That' s all, folks :)

