package com.miami.romeo.main.tree;

import java.awt.*;
import javax.swing.*;
import javax.swing.tree.*;

import com.miami.romeo.config.*;

/**
 * JTree renderer
 */
public class TreeNodeRenderer extends DefaultTreeCellRenderer {
  Color background;

  Utils utils = new Utils();

  ImageIcon folder, folder_open, romIcon, bookmarkIcon, availableIcon,
            othersIcon, historyIcon, manufacturerIcon, workingIcon,
            nonWorkingIcon, /*mameIcon,*/ neogeoIcon, folderRedIcon;

  public TreeNodeRenderer(Color background) {
    this.background = background;

    folder = utils.createImageIcon16x16("Folder.png");
    folder_open = utils.createImageIcon16x16("Folder_open.png");
    romIcon = utils.createImageIcon16x16("Rom.png");
    bookmarkIcon = utils.createImageIcon16x16("Bookmark_folder.png");
    availableIcon = utils.createImageIcon16x16("Hdd.png");
    othersIcon = utils.createImageIcon16x16("Node.png");
    workingIcon = utils.createImageIcon16x16("Ok.png");
    nonWorkingIcon = utils.createImageIcon16x16("Trash.png");
    neogeoIcon = utils.createImageIcon16x16("neogeo.png");
    historyIcon = utils.createImageIcon16x16("History.png");
    manufacturerIcon = utils.createImageIcon16x16("Manufacturer.png");
    /*mameIcon = utils.createImageIcon16x16("Mamedk.png");*/
    folderRedIcon = utils.createImageIcon16x16("Folder_red.png");
  }

  public Component getTreeCellRendererComponent(JTree tree, Object value,
                                                boolean sel, boolean expanded,
                                                boolean leaf, int row,
                                                boolean hasFocus) {

    super.getTreeCellRendererComponent(tree, value, sel,
                                       expanded, leaf, row,
                                       hasFocus);

    String stringValue = tree.convertValueToText(value, sel,
			                         expanded, leaf,
						 row, hasFocus);

    setText(stringValue);

    if (sel) {
      setForeground(getTextSelectionColor());
    } else {
      setBackgroundNonSelectionColor(background);
      //setForeground(getTextNonSelectionColor());
      int r = background.getRed();
      int g = background.getGreen();
      int b = background.getBlue();

      if(r > 240 || g > 240) {
        setForeground(Color.black);
      } else {
        setForeground(Color.white);
      }
    }

    boolean treeIsEnabled = tree.isEnabled();
    boolean nodeIsEnabled = ((TreeNode)value).isEnabled();
    boolean isEnabled = (treeIsEnabled && nodeIsEnabled);
    setEnabled(isEnabled);

    if (isEnabled) {
      selected = sel;
      if (leaf) {
        if (row == 1)
          setIcon(romIcon);
        else if (row == 2)
          setIcon(bookmarkIcon);
        else if (row == 3)
          setIcon(availableIcon);
	else if (row == 4)
          setIcon(othersIcon);
	else if (row == 5)
          setIcon(othersIcon);
	else if (row == 6)
          setIcon(workingIcon);
	else if (row == 7)
          setIcon(nonWorkingIcon);
        // Neo-Geo
	else if (row == 8)
          setIcon(neogeoIcon);
	// CPS1
        else if (row == 9)
          setIcon(folderRedIcon);
	// CPS2
	else if (row == 10)
	  setIcon(folderRedIcon);
	else if (isYearNode(value))
	  setIcon(historyIcon);
	else if (isCategoryNode(value))
	  setIcon(folderRedIcon);
        /*
	else if (isTopRatedGamesNode(value))
	  setIcon(mameIcon);
        */
        else
	  setIcon(manufacturerIcon);
      }
      else {
        setLeafIcon(othersIcon);
        setOpenIcon(folder_open);
        setClosedIcon(folder);
      }
    }
    else {
      selected = false;
      if (leaf) {
        if (row == 1)
          setLeafIcon(new GraydIcon(romIcon));
        else if (row == 2)
          setLeafIcon(new GraydIcon(bookmarkIcon));
        else if (row == 3)
          setLeafIcon(new GraydIcon(availableIcon));
	else if (row == 4)
          setLeafIcon(new GraydIcon(othersIcon));
	else if (row == 5)
          setLeafIcon(new GraydIcon(othersIcon));
	else if (row == 6)
          setLeafIcon(new GraydIcon(workingIcon));
	else if (row == 7)
          setLeafIcon(new GraydIcon(nonWorkingIcon));
        // Neo-Geo
	else if (row == 8)
          setLeafIcon(new GraydIcon(neogeoIcon));
	// CPS1
        else if (row == 9)
          setLeafIcon(new GraydIcon(folderRedIcon));
	// CPS2
	else if (row == 10)
	  setLeafIcon(new GraydIcon(folderRedIcon));
	else if (isYearNode(value))
	  setLeafIcon(new GraydIcon(historyIcon));
	else if (isCategoryNode(value))
	  setLeafIcon(new GraydIcon(othersIcon));
        /*
	else if (isTopRatedGamesNode(value))
	  setLeafIcon(new GraydIcon(mameIcon));
        */
        else
	  setLeafIcon(new GraydIcon(manufacturerIcon));
      }
      else {
        setLeafIcon(new GraydIcon(othersIcon));
        setOpenIcon(new GraydIcon(folder_open));
        setClosedIcon(new GraydIcon(folder));
      }
    }

    return this;
   }

   //****** Methods to retrieve type of selected node
   protected boolean isYearNode(Object value) {
     DefaultMutableTreeNode node = (DefaultMutableTreeNode)value;
     RomsInfo romsinfo = (RomsInfo)(node.getUserObject());
     String title = romsinfo.toString();
     if (title.startsWith("19") || title.startsWith("20")) {
       return true;
     }

     return false;
   }

   protected boolean isCategoryNode(Object value) {
     DefaultMutableTreeNode node = (DefaultMutableTreeNode)value;
     RomsInfo romsinfo = (RomsInfo)(node.getUserObject());
     String title = romsinfo.toString();
     if (title.startsWith("Adult") || title.startsWith("Adventure") ||
         title.startsWith("Breakout") || title.startsWith("Cards") ||
	 title.startsWith("Diagnostic") || title.startsWith("Fighter") ||
         title.startsWith("Gambling") || title.startsWith("Gun") ||
	 title.startsWith("Mahjong") || title.startsWith("Maze") ||
	 title.startsWith("Misc") || title.startsWith("Pinball") ||
	 title.startsWith("Platform") ||
	 title.startsWith("Puzzle") || title.startsWith("Quiz") ||
	 title.startsWith("Racing") || title.startsWith("Shooter") ||
	 title.startsWith("Sports") || title.equals("Wrestling") ||
	 title.equals("[Unknown]")) {
       return true;
     }

     return false;
   }

   /*
   protected boolean isTopRatedGamesNode(Object value) {
     DefaultMutableTreeNode node = (DefaultMutableTreeNode)value;
     RomsInfo romsinfo = (RomsInfo)(node.getUserObject());
     String title = romsinfo.toString();
     if (title.equals("top Adult games") || title.equals("top Breakout games") ||
         title.equals("top Fighter games") || title.equals("top Gun games") ||
	 title.equals("top Mahjong games") || title.equals("top Maze games") ||
	 title.equals("top Misc games") || title.equals("top Platform games") ||
	 title.equals("top Puzzle games") || title.equals("top Quiz games") ||
	 title.equals("top Racing games") || title.equals("top Shooter games") ||
	 title.equals("top Sports games")) {
       return true;
     }

     return false;
   }
   */
 }
